# coding: utf-8

# flake8: noqa
"""
Snowflake Alert API.

The Snowflake Alert API is a REST API that you can use to access, update, and perform certain actions on Alert resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.alert._generated.models.alert import Alert
from snowflake.core.alert._generated.models.alert_clone import AlertClone
from snowflake.core.alert._generated.models.cron_schedule import CronSchedule
from snowflake.core.alert._generated.models.error_response import ErrorResponse
from snowflake.core.alert._generated.models.minutes_schedule import MinutesSchedule
from snowflake.core.alert._generated.models.point_of_time import PointOfTime
from snowflake.core.alert._generated.models.point_of_time_offset import PointOfTimeOffset
from snowflake.core.alert._generated.models.point_of_time_statement import PointOfTimeStatement
from snowflake.core.alert._generated.models.point_of_time_timestamp import PointOfTimeTimestamp
from snowflake.core.alert._generated.models.schedule import Schedule
from snowflake.core.alert._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.alert._generated.models.success_response import SuccessResponse
from snowflake.core.alert._generated.models.tag_assignment import TagAssignment
from snowflake.core.alert._generated.models.tag_reference import TagReference

__all__ = [
    "Alert",
    "AlertClone",
    "CronSchedule",
    "ErrorResponse",
    "MinutesSchedule",
    "PointOfTime",
    "PointOfTimeOffset",
    "PointOfTimeStatement",
    "PointOfTimeTimestamp",
    "Schedule",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.


import pytest

from snowflake.core.database import DatabaseCollection  # noqa: F401
from snowflake.core.task import TaskCollection

from ..fixtures.pre_checks import my_integration_exists  # noqa: F401 # pylint: disable=unused-import


@pytest.fixture(scope="module")
def tasks(schema) -> TaskCollection:
    return schema.tasks



import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import ConflictError, NotFoundError
from snowflake.core.role import Role
from tests.utils import random_string


@pytest.mark.use_accountadmin
def test_create(roles, root, session):
    role_name = random_string(4, "test_create_role_")
    try:
        test_role = Role(name=role_name, comment="test_comment")
        created_role = roles.create(test_role)
        assert created_role.name == role_name

        # create role with already existing name with mode or_replace
        test_role.comment = "new comment"
        roles.create(test_role, mode=CreateMode.or_replace)
        replaced_roles = list(roles.iter(like=role_name))
        assert len(replaced_roles) == 1
        replaced_role = replaced_roles[0]
        assert replaced_role is not None
        assert replaced_role.name == role_name.upper()
        assert replaced_role.comment == "new comment"

        with pytest.raises(ConflictError):
            # throws error if test_role is already present.
            roles.create(test_role, mode=CreateMode.error_if_exists)

        # will succeed without any errors.
        roles.create(test_role, mode=CreateMode.if_not_exists)

    finally:
        roles[role_name].drop(if_exists=True)


@pytest.mark.use_accountadmin
def test_drop(roles, root, session):
    role_name = random_string(4, "test_drop_role_")
    try:
        test_role = Role(name=role_name, comment="test drop role")

        created_role = roles.create(test_role)
        assert created_role.name == role_name
        roles[role_name].drop()
        created_role = None

        with pytest.raises(NotFoundError):
            # throws error as test_role is already dropped
            roles[role_name].drop()

        roles[role_name].drop(if_exists=True)
    finally:
        roles[role_name].drop(if_exists=True)



"""
Snowflake Event Table API.

The Snowflake Event Table API is a REST API that you can use to access, update, and perform certain actions on Event Table resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr

from snowflake.core.event_table._generated.models.event_table_column import EventTableColumn, EventTableColumnModel


class EventTable(BaseModel):
    """A model object representing the EventTable resource.

    Constructs an object of type EventTable with the provided properties.

    Parameters
    __________
    name : str
        Name of the event table
    cluster_by : list[str], optional
        Cluster key column(s) or expression
    data_retention_time_in_days : int, optional
        number of days to retain the old version of deleted/updated data
    max_data_extension_time_in_days : int, optional
        Maximum number of days to extend data retention beyond the retention period to prevent a stream becoming stale.
    change_tracking : bool, optional
        True if change tracking is enabled, allowing streams and CHANGES to be used on the entity.
    default_ddl_collation : str, optional
        Collation that is used for all the new columns created by the DDL statements (if not specified)
    comment : str, optional
        user comment associated to an object in the dictionary
    created_on : datetime, optional
        Date and time when the event table was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the event table is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the event table is stored — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the event table — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the event table — **Read-only:** *any user-provided value will be ignored.*
    rows : int, optional
        Number of rows in the table — **Read-only:** *any user-provided value will be ignored.*
    bytes : int, optional
        Number of bytes that will be scanned if the entire table is scanned in a query.Note that this number may be different than the number of actual physical bytes stored on-disk for the table — **Read-only:** *any user-provided value will be ignored.*
    automatic_clustering : bool, optional
        If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the table — **Read-only:** *any user-provided value will be ignored.*
    search_optimization : bool, optional
        If ON, the table has the search optimization service enabled — **Read-only:** *any user-provided value will be ignored.*
    search_optimization_progress : int, optional
        Percentage of the table that has been optimized for search — **Read-only:** *any user-provided value will be ignored.*
    search_optimization_bytes : int, optional
        Number of additional bytes of storage that the search optimization service consumes for this table — **Read-only:** *any user-provided value will be ignored.*
    columns : list[EventTableColumn], optional
        **Read-only:** *any user-provided value will be ignored.*
    """

    name: StrictStr

    cluster_by: Optional[List[StrictStr]] = None

    data_retention_time_in_days: Optional[StrictInt] = None

    max_data_extension_time_in_days: Optional[StrictInt] = None

    change_tracking: Optional[StrictBool] = None

    default_ddl_collation: Optional[StrictStr] = None

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    rows: Optional[StrictInt] = None

    bytes: Optional[StrictInt] = None

    automatic_clustering: Optional[StrictBool] = None

    search_optimization: Optional[StrictBool] = None

    search_optimization_progress: Optional[StrictInt] = None

    search_optimization_bytes: Optional[StrictInt] = None

    columns: Optional[List[EventTableColumn]] = None

    __properties = [
        "name",
        "cluster_by",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "change_tracking",
        "default_ddl_collation",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
        "rows",
        "bytes",
        "automatic_clustering",
        "search_optimization",
        "search_optimization_progress",
        "search_optimization_bytes",
        "columns",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EventTable:
        """Create an instance of EventTable from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "owner",
                    "owner_role_type",
                    "rows",
                    "bytes",
                    "automatic_clustering",
                    "search_optimization",
                    "search_optimization_progress",
                    "search_optimization_bytes",
                    "columns",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict["columns"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> EventTable:
        """Create an instance of EventTable from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return EventTable.model_validate(obj)

        _obj = EventTable.model_validate(
            {
                "name": obj.get("name"),
                "cluster_by": obj.get("cluster_by"),
                "data_retention_time_in_days": obj.get("data_retention_time_in_days"),
                "max_data_extension_time_in_days": obj.get("max_data_extension_time_in_days"),
                "change_tracking": obj.get("change_tracking"),
                "default_ddl_collation": obj.get("default_ddl_collation"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "rows": obj.get("rows"),
                "bytes": obj.get("bytes"),
                "automatic_clustering": obj.get("automatic_clustering"),
                "search_optimization": obj.get("search_optimization"),
                "search_optimization_progress": obj.get("search_optimization_progress"),
                "search_optimization_bytes": obj.get("search_optimization_bytes"),
                "columns": [EventTableColumn.from_dict(_item) for _item in obj.get("columns")]
                if obj.get("columns") is not None
                else None,
            }
        )

        return _obj


class EventTableModel:
    def __init__(
        self,
        name: str,
        # optional properties
        cluster_by: Optional[list[str]] = None,
        data_retention_time_in_days: Optional[int] = None,
        max_data_extension_time_in_days: Optional[int] = None,
        change_tracking: Optional[bool] = None,
        default_ddl_collation: Optional[str] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        rows: Optional[int] = None,
        bytes: Optional[int] = None,
        automatic_clustering: Optional[bool] = None,
        search_optimization: Optional[bool] = None,
        search_optimization_progress: Optional[int] = None,
        search_optimization_bytes: Optional[int] = None,
        columns: Optional[list[EventTableColumn]] = None,
    ):
        """A model object representing the EventTable resource.

        Constructs an object of type EventTable with the provided properties.

        Parameters
        __________
        name : str
            Name of the event table
        cluster_by : list[str], optional
            Cluster key column(s) or expression
        data_retention_time_in_days : int, optional
            number of days to retain the old version of deleted/updated data
        max_data_extension_time_in_days : int, optional
            Maximum number of days to extend data retention beyond the retention period to prevent a stream becoming stale.
        change_tracking : bool, optional
            True if change tracking is enabled, allowing streams and CHANGES to be used on the entity.
        default_ddl_collation : str, optional
            Collation that is used for all the new columns created by the DDL statements (if not specified)
        comment : str, optional
            user comment associated to an object in the dictionary
        created_on : datetime, optional
            Date and time when the event table was created.
        database_name : str, optional
            Database in which the event table is stored
        schema_name : str, optional
            Schema in which the event table is stored
        owner : str, optional
            Role that owns the event table
        owner_role_type : str, optional
            The type of role that owns the event table
        rows : int, optional
            Number of rows in the table.
        bytes : int, optional
            Number of bytes that will be scanned if the entire table is scanned in a query.Note that this number may be different than the number of actual physical bytes stored on-disk for the table
        automatic_clustering : bool, optional
            If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the table.
        search_optimization : bool, optional
            If ON, the table has the search optimization service enabled
        search_optimization_progress : int, optional
            Percentage of the table that has been optimized for search
        search_optimization_bytes : int, optional
            Number of additional bytes of storage that the search optimization service consumes for this table
        columns : list[EventTableColumn], optional
        """
        self.name = name
        self.cluster_by = cluster_by
        self.data_retention_time_in_days = data_retention_time_in_days
        self.max_data_extension_time_in_days = max_data_extension_time_in_days
        self.change_tracking = change_tracking
        self.default_ddl_collation = default_ddl_collation
        self.comment = comment
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.rows = rows
        self.bytes = bytes
        self.automatic_clustering = automatic_clustering
        self.search_optimization = search_optimization
        self.search_optimization_progress = search_optimization_progress
        self.search_optimization_bytes = search_optimization_bytes
        self.columns = columns

    __properties = [
        "name",
        "cluster_by",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "change_tracking",
        "default_ddl_collation",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
        "rows",
        "bytes",
        "automatic_clustering",
        "search_optimization",
        "search_optimization_progress",
        "search_optimization_bytes",
        "columns",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return EventTable(
            name=self.name,
            cluster_by=self.cluster_by,
            data_retention_time_in_days=self.data_retention_time_in_days,
            max_data_extension_time_in_days=self.max_data_extension_time_in_days,
            change_tracking=self.change_tracking,
            default_ddl_collation=self.default_ddl_collation,
            comment=self.comment,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            rows=self.rows,
            bytes=self.bytes,
            automatic_clustering=self.automatic_clustering,
            search_optimization=self.search_optimization,
            search_optimization_progress=self.search_optimization_progress,
            search_optimization_bytes=self.search_optimization_bytes,
            columns=[x._to_model() for x in self.columns] if self.columns is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> EventTableModel:
        return EventTableModel(
            name=model.name,
            cluster_by=model.cluster_by,
            data_retention_time_in_days=model.data_retention_time_in_days,
            max_data_extension_time_in_days=model.max_data_extension_time_in_days,
            change_tracking=model.change_tracking,
            default_ddl_collation=model.default_ddl_collation,
            comment=model.comment,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            rows=model.rows,
            bytes=model.bytes,
            automatic_clustering=model.automatic_clustering,
            search_optimization=model.search_optimization,
            search_optimization_progress=model.search_optimization_progress,
            search_optimization_bytes=model.search_optimization_bytes,
            columns=[EventTableColumnModel._from_model(x) for x in model.columns] if model.columns else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a EventTable.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the EventTable object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> EventTableModel:
        """Create an instance of EventTable from a dict.

        This method constructs a EventTable object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        EventTable
            A EventTable object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(EventTable.from_dict(obj))


EventTable._model_class = EventTableModel



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_password_policy_template


def test_drop(password_policies):
    prefix = "test_password_policy_drop_"
    pre_create_count = len(list(password_policies.iter(like=prefix + "%")))

    name = random_string(10, prefix)
    password_policy_handle = password_policies[name]

    password_policy = copy.deepcopy(test_password_policy_template)
    password_policy.name = name
    password_policies.create(password_policy)

    created_count = len(list(password_policies.iter(like=prefix + "%")))
    assert pre_create_count + 1 == created_count

    password_policy_handle.drop()

    after_drop_count = len(list(password_policies.iter(like=prefix + "%")))
    assert pre_create_count == after_drop_count

    with pytest.raises(NotFoundError):
        password_policy_handle.drop()
        password_policy_handle.drop(if_exists=False)

    password_policy_handle.drop(if_exists=True)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.network_policy import NetworkPolicy, NetworkPolicyCollection, NetworkPolicyResource
from snowflake.core.network_policy._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def network_policies(fake_root):
    return NetworkPolicyCollection(fake_root)


@pytest.fixture
def network_policy(network_policies):
    return network_policies["my_policy"]


def test_create_network_policy(fake_root, network_policies):
    args = (fake_root, "POST", BASE_URL + "/network-policies")
    kwargs = extra_params(query_params=[], body={"name": "my_policy"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        np_res = network_policies.create(NetworkPolicy(name="my_policy"))
        assert isinstance(np_res, NetworkPolicyResource)
        assert np_res.name == "my_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_policies.create_async(NetworkPolicy(name="my_policy"))
        assert isinstance(op, PollingOperation)
        et_res = op.result()
        assert et_res.name == "my_policy"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_network_policy(fake_root, network_policies):
    args = (fake_root, "GET", BASE_URL + "/network-policies")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        network_policies.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = network_policies.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_network_policy(fake_root, network_policy):
    from snowflake.core.network_policy._generated.models import NetworkPolicy as NetworkPolicyModel

    model = NetworkPolicyModel(name="my_policy")
    args = (fake_root, "GET", BASE_URL + "/network-policies/my_policy")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        network_policy.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = network_policy.fetch_async()
        assert isinstance(op, PollingOperation)
        tab = op.result()
        assert tab.to_dict() == NetworkPolicy(name="my_policy").to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_network_policy(fake_root, network_policy):
    args = (fake_root, "DELETE", BASE_URL + "/network-policies/my_policy")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        network_policy.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_policy.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, network_policy, tag):
    args = (fake_root, "POST", BASE_URL + "/network-policies/my_policy:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        network_policy.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_policy.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, network_policy, tag):
    args = (fake_root, "POST", BASE_URL + "/network-policies/my_policy:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        network_policy.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = network_policy.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, network_policy):
    args = (fake_root, "GET", BASE_URL + "/network-policies/my_policy:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert network_policy.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = network_policy.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from snowflake.core.exceptions import APIError, ConflictError
from tests.integ.account.utils import assert_account_existence, cleanup_accounts_completely, create_account

from ..utils import backup_role


pytestmark = [pytest.mark.internal_only, pytest.mark.skip_notebook]


def test_undrop_account(temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor):
    with backup_role(temp_customer_account_cursor):
        try:
            temp_customer_account_cursor.execute("use role orgadmin;")
            # test happy path
            account = create_account(temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor)

            # undrop existing account
            with pytest.raises(ConflictError):
                temp_customer_account_accounts[account.name].undrop()

            # account deleted
            temp_customer_account_accounts[account.name].drop(grace_period_in_days=4)
            assert_account_existence(temp_customer_account_accounts, account.name, should_exist=False)

            # undrop deleted account
            temp_customer_account_accounts[account.name].undrop()
            assert_account_existence(temp_customer_account_accounts, account.name)

            # account already restored account
            with pytest.raises(ConflictError):
                temp_customer_account_accounts[account.name].undrop()

            # undrop random account that does not exist
            with pytest.raises(APIError):
                temp_customer_account_accounts["RANDOM"].undrop()

            # cleanup
            temp_customer_account_accounts[account.name].drop(grace_period_in_days=4)
        finally:
            # cleanup
            cleanup_accounts_completely(sf_cursor)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.tag import TagValue
from snowflake.core.user import Securable, User, UserCollection, UserResource
from snowflake.core.user._generated import TagAssignment, TagReference

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def users(fake_root):
    return UserCollection(fake_root)


@pytest.fixture
def user(users):
    return users["admin"]


def test_create_user(fake_root, users):
    args = (fake_root, "POST", BASE_URL + "/users?createMode=errorIfExists")
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists")], body={"name": "admin", "default_secondary_roles": "ALL"}
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user_res = users.create(User(name="admin"))
        assert isinstance(user_res, UserResource)
        assert user_res.name == "admin"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = users.create_async(User(name="admin"))
        assert isinstance(op, PollingOperation)
        user_res = op.result()
        assert user_res.name == "admin"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_user(fake_root, users):
    args = (fake_root, "GET", BASE_URL + "/users")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        users.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = users.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_user(fake_root, user):
    args = (fake_root, "PUT", BASE_URL + "/users/admin")
    kwargs = extra_params(body={"name": "admin", "default_secondary_roles": "ALL"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.create_or_alter(User(name="admin"))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.create_or_alter_async(User(name="admin"))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_user(fake_root, user):
    from snowflake.core.user._generated.models import User as UserModel

    model = UserModel(name="admin")
    args = (fake_root, "GET", BASE_URL + "/users/admin")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        user.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = user.fetch_async()
        assert isinstance(op, PollingOperation)
        user = op.result()
        assert user.to_dict() == User(name="admin").to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_user(fake_root, user):
    args = (fake_root, "DELETE", BASE_URL + "/users/admin")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_grant_role_user(fake_root, user):
    args = (fake_root, "POST", BASE_URL + "/users/admin/grants")
    kwargs = extra_params(body={"securable": {"name": "test_role"}, "securable_type": "ROLE"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.grant_role("ROLE", Securable(name="test_role"))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.grant_role_async("ROLE", Securable(name="test_role"))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_revoke_role_user(fake_root, user):
    args = (fake_root, "POST", BASE_URL + "/users/admin/grants:revoke")
    kwargs = extra_params(body={"securable": {"name": "test_role"}, "securable_type": "ROLE"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.revoke_role("ROLE", Securable(name="test_role"))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.revoke_role_async("ROLE", Securable(name="test_role"))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_grants_to_user(fake_root, user):
    args = (fake_root, "GET", BASE_URL + "/users/admin/grants")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = user.iter_grants_to()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = user.iter_grants_to_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, user, tag):
    args = (fake_root, "POST", BASE_URL + "/users/admin:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, user, tag):
    args = (fake_root, "POST", BASE_URL + "/users/admin:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        user.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = user.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, user):
    args = (fake_root, "GET", BASE_URL + "/users/admin:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert user.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = user.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Snowflake Warehouse API.

The Snowflake Warehouse API is a REST API that you can use to access, customize and manage virtual warehouse in a Snowflake account.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing_extensions import Annotated


class Warehouse(BaseModel):
    """A model object representing the Warehouse resource.

    Constructs an object of type Warehouse with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    warehouse_type : str, optional
        Type of warehouse, possible types: STANDARD, SNOWPARK-OPTIMIZED
    warehouse_size : str, optional
        Size of warehouse, possible sizes: XSMALL, SMALL, MEDIUM, LARGE, XLARGE, XXLARGE, XXXLARGE, X4LARGE, X5LARGE, X6LARGE
    wait_for_completion : str, optional
        When resizing a warehouse, you can use this parameter to block the return of the ALTER WAREHOUSE command until the resize has finished provisioning all its compute resources
    max_cluster_count : int, optional
        Specifies the maximum number of clusters for a multi-cluster warehouse
    min_cluster_count : int, optional
        Specifies the minimum number of clusters for a multi-cluster warehouse
    scaling_policy : str, optional
        Scaling policy of warehouse, possible scaling policies: STANDARD, ECONOMY
    auto_suspend : int, optional
        time in seconds before auto suspend
    auto_resume : str, optional
        Specifies whether to automatically resume a warehouse when a SQL statement is submitted to it
    initially_suspended : str, optional
        Specifies whether the warehouse is created initially in the Suspended state
    resource_monitor : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    comment : str, optional
        Specifies a comment for the warehouse
    enable_query_acceleration : str, optional
        Specifies whether to enable the query acceleration service for queries that rely on this warehouse for compute resources
    query_acceleration_max_scale_factor : int, optional
        Specifies the maximum scale factor for leasing compute resources for query acceleration. The scale factor is used as a multiplier based on warehouse size
    max_concurrency_level : int, optional
        Object parameter that specifies the concurrency level for SQL statements executed by a warehouse cluster
    statement_queued_timeout_in_seconds : int, optional
        Object parameter that specifies the time, in seconds, a SQL statement can be queued on a warehouse before it is canceled by the system
    statement_timeout_in_seconds : int, optional
        Object parameter that specifies the time, in seconds, after which a running SQL statement  is canceled by the system
    type : str, optional
        [Deprecated] Type of warehouse, possible types: STANDARD, SNOWPARK-OPTIMIZED
    size : str, optional
        [Deprecated] names of size: X-Small, Small, Medium, Large, X-Large, 2X-Large, 3X-Large, 4X-Large, 5X-Large, 6X-Large
    state : str, optional
        The state of warehouse, possible states: STARTED, STARTING, DYNAMIC, SUSPENDED, RESIZING, RESUMING, SUSPENDING — **Read-only:** *any user-provided value will be ignored.*
    started_clusters : int, optional
        Number of clusters currently started — **Read-only:** *any user-provided value will be ignored.*
    running : int, optional
        Number of SQL statements that are being executed by the warehouse — **Read-only:** *any user-provided value will be ignored.*
    queued : int, optional
        Number of SQL statements that are queued for the warehouse — **Read-only:** *any user-provided value will be ignored.*
    is_default : bool, optional
        Whether the warehouse is the default for the current user — **Read-only:** *any user-provided value will be ignored.*
    is_current : bool, optional
        Whether the warehouse is in use for the session. Only one warehouse can be in use at a time for a session.  To specify or change the warehouse for a session, use the USE WAREHOUSE command — **Read-only:** *any user-provided value will be ignored.*
    available : str, optional
        Percentage of the warehouse compute resources that are provisioned and available — **Read-only:** *any user-provided value will be ignored.*
    provisioning : str, optional
        Percentage of the warehouse compute resources that are in the process of provisioning — **Read-only:** *any user-provided value will be ignored.*
    quiescing : str, optional
        Percentage of the warehouse compute resources that are executing SQL statements,  but will be shut down once the queries complete — **Read-only:** *any user-provided value will be ignored.*
    other : str, optional
        Percentage of the warehouse compute resources that are in a state other than available,  provisioning, or quiescing — **Read-only:** *any user-provided value will be ignored.*
    created_on : datetime, optional
        Date and time when the warehouse was created — **Read-only:** *any user-provided value will be ignored.*
    resumed_on : datetime, optional
        Date and time when the warehouse was last started or restarted — **Read-only:** *any user-provided value will be ignored.*
    updated_on : datetime, optional
        Date and time when the warehouse was last updated,  which includes changing any of the properties of the warehouse or changing the state (STARTED, SUSPENDED, RESIZING) of the warehouse — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the warehouse — **Read-only:** *any user-provided value will be ignored.*
    budget : str, optional
        Comment representing budget for warehouse — **Read-only:** *any user-provided value will be ignored.*
    kind : str, optional
        **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the object — **Read-only:** *any user-provided value will be ignored.*
    warehouse_credit_limit : int, optional
        Credit limit that are can be executed by the warehouse.
    target_statement_size : str, optional
        Names of size: X-Small, Small, Medium, Large, X-Large, 2X-Large, 3X-Large, 4X-Large, 5X-Large, 6X-Large
    """

    name: Annotated[str, Field(strict=True)]

    warehouse_type: Optional[StrictStr] = None

    warehouse_size: Optional[StrictStr] = None

    wait_for_completion: Optional[StrictStr] = None

    max_cluster_count: Optional[StrictInt] = None

    min_cluster_count: Optional[StrictInt] = None

    scaling_policy: Optional[StrictStr] = None

    auto_suspend: Optional[StrictInt] = None

    auto_resume: Optional[StrictStr] = None

    initially_suspended: Optional[StrictStr] = None

    resource_monitor: Optional[Annotated[str, Field(strict=True)]] = None

    comment: Optional[StrictStr] = None

    enable_query_acceleration: Optional[StrictStr] = None

    query_acceleration_max_scale_factor: Optional[StrictInt] = None

    max_concurrency_level: Optional[StrictInt] = None

    statement_queued_timeout_in_seconds: Optional[StrictInt] = None

    statement_timeout_in_seconds: Optional[StrictInt] = None

    type: Optional[StrictStr] = None

    size: Optional[StrictStr] = None

    state: Optional[StrictStr] = None

    started_clusters: Optional[StrictInt] = None

    running: Optional[StrictInt] = None

    queued: Optional[StrictInt] = None

    is_default: Optional[StrictBool] = None

    is_current: Optional[StrictBool] = None

    available: Optional[StrictStr] = None

    provisioning: Optional[StrictStr] = None

    quiescing: Optional[StrictStr] = None

    other: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    resumed_on: Optional[datetime] = None

    updated_on: Optional[datetime] = None

    owner: Optional[StrictStr] = None

    budget: Optional[StrictStr] = None

    kind: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    warehouse_credit_limit: Optional[StrictInt] = None

    target_statement_size: Optional[StrictStr] = None

    __properties = [
        "name",
        "warehouse_type",
        "warehouse_size",
        "wait_for_completion",
        "max_cluster_count",
        "min_cluster_count",
        "scaling_policy",
        "auto_suspend",
        "auto_resume",
        "initially_suspended",
        "resource_monitor",
        "comment",
        "enable_query_acceleration",
        "query_acceleration_max_scale_factor",
        "max_concurrency_level",
        "statement_queued_timeout_in_seconds",
        "statement_timeout_in_seconds",
        "type",
        "size",
        "state",
        "started_clusters",
        "running",
        "queued",
        "is_default",
        "is_current",
        "available",
        "provisioning",
        "quiescing",
        "other",
        "created_on",
        "resumed_on",
        "updated_on",
        "owner",
        "budget",
        "kind",
        "owner_role_type",
        "warehouse_credit_limit",
        "target_statement_size",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("wait_for_completion")
    def wait_for_completion_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("true", "false"):
            raise ValueError("must validate the enum values ('true','false')")
        return v

    @field_validator("auto_resume")
    def auto_resume_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("true", "false"):
            raise ValueError("must validate the enum values ('true','false')")
        return v

    @field_validator("initially_suspended")
    def initially_suspended_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("true", "false"):
            raise ValueError("must validate the enum values ('true','false')")
        return v

    @field_validator("resource_monitor")
    def resource_monitor_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("enable_query_acceleration")
    def enable_query_acceleration_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("true", "false"):
            raise ValueError("must validate the enum values ('true','false')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Warehouse:
        """Create an instance of Warehouse from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "state",
                    "started_clusters",
                    "running",
                    "queued",
                    "is_default",
                    "is_current",
                    "available",
                    "provisioning",
                    "quiescing",
                    "other",
                    "created_on",
                    "resumed_on",
                    "updated_on",
                    "owner",
                    "budget",
                    "kind",
                    "owner_role_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Warehouse:
        """Create an instance of Warehouse from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Warehouse.model_validate(obj)

        _obj = Warehouse.model_validate(
            {
                "name": obj.get("name"),
                "warehouse_type": obj.get("warehouse_type"),
                "warehouse_size": obj.get("warehouse_size"),
                "wait_for_completion": obj.get("wait_for_completion"),
                "max_cluster_count": obj.get("max_cluster_count"),
                "min_cluster_count": obj.get("min_cluster_count"),
                "scaling_policy": obj.get("scaling_policy"),
                "auto_suspend": obj.get("auto_suspend"),
                "auto_resume": obj.get("auto_resume"),
                "initially_suspended": obj.get("initially_suspended"),
                "resource_monitor": obj.get("resource_monitor"),
                "comment": obj.get("comment"),
                "enable_query_acceleration": obj.get("enable_query_acceleration"),
                "query_acceleration_max_scale_factor": obj.get("query_acceleration_max_scale_factor"),
                "max_concurrency_level": obj.get("max_concurrency_level"),
                "statement_queued_timeout_in_seconds": obj.get("statement_queued_timeout_in_seconds"),
                "statement_timeout_in_seconds": obj.get("statement_timeout_in_seconds"),
                "type": obj.get("type"),
                "size": obj.get("size"),
                "state": obj.get("state"),
                "started_clusters": obj.get("started_clusters"),
                "running": obj.get("running"),
                "queued": obj.get("queued"),
                "is_default": obj.get("is_default"),
                "is_current": obj.get("is_current"),
                "available": obj.get("available"),
                "provisioning": obj.get("provisioning"),
                "quiescing": obj.get("quiescing"),
                "other": obj.get("other"),
                "created_on": obj.get("created_on"),
                "resumed_on": obj.get("resumed_on"),
                "updated_on": obj.get("updated_on"),
                "owner": obj.get("owner"),
                "budget": obj.get("budget"),
                "kind": obj.get("kind"),
                "owner_role_type": obj.get("owner_role_type"),
                "warehouse_credit_limit": obj.get("warehouse_credit_limit"),
                "target_statement_size": obj.get("target_statement_size"),
            }
        )

        return _obj


class WarehouseModel:
    def __init__(
        self,
        name: str,
        # optional properties
        warehouse_type: Optional[str] = None,
        warehouse_size: Optional[str] = None,
        wait_for_completion: Optional[str] = None,
        max_cluster_count: Optional[int] = None,
        min_cluster_count: Optional[int] = None,
        scaling_policy: Optional[str] = None,
        auto_suspend: Optional[int] = None,
        auto_resume: Optional[str] = None,
        initially_suspended: Optional[str] = None,
        resource_monitor: Optional[str] = None,
        comment: Optional[str] = None,
        enable_query_acceleration: Optional[str] = None,
        query_acceleration_max_scale_factor: Optional[int] = None,
        max_concurrency_level: Optional[int] = None,
        statement_queued_timeout_in_seconds: Optional[int] = None,
        statement_timeout_in_seconds: Optional[int] = None,
        type: Optional[str] = None,
        size: Optional[str] = None,
        state: Optional[str] = None,
        started_clusters: Optional[int] = None,
        running: Optional[int] = None,
        queued: Optional[int] = None,
        is_default: Optional[bool] = None,
        is_current: Optional[bool] = None,
        available: Optional[str] = None,
        provisioning: Optional[str] = None,
        quiescing: Optional[str] = None,
        other: Optional[str] = None,
        created_on: Optional[datetime] = None,
        resumed_on: Optional[datetime] = None,
        updated_on: Optional[datetime] = None,
        owner: Optional[str] = None,
        budget: Optional[str] = None,
        kind: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        warehouse_credit_limit: Optional[int] = None,
        target_statement_size: Optional[str] = None,
    ):
        """A model object representing the Warehouse resource.

        Constructs an object of type Warehouse with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        warehouse_type : str, optional
            Type of warehouse, possible types: STANDARD, SNOWPARK-OPTIMIZED
        warehouse_size : str, optional
            Size of warehouse, possible sizes: XSMALL, SMALL, MEDIUM, LARGE, XLARGE, XXLARGE, XXXLARGE, X4LARGE, X5LARGE, X6LARGE
        wait_for_completion : str, optional
            When resizing a warehouse, you can use this parameter to block the return of the ALTER WAREHOUSE command until the resize has finished provisioning all its compute resources
        max_cluster_count : int, optional
            Specifies the maximum number of clusters for a multi-cluster warehouse
        min_cluster_count : int, optional
            Specifies the minimum number of clusters for a multi-cluster warehouse
        scaling_policy : str, optional
            Scaling policy of warehouse, possible scaling policies: STANDARD, ECONOMY
        auto_suspend : int, optional
            time in seconds before auto suspend
        auto_resume : str, optional
            Specifies whether to automatically resume a warehouse when a SQL statement is submitted to it
        initially_suspended : str, optional
            Specifies whether the warehouse is created initially in the Suspended state
        resource_monitor : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        comment : str, optional
            Specifies a comment for the warehouse
        enable_query_acceleration : str, optional
            Specifies whether to enable the query acceleration service for queries that rely on this warehouse for compute resources
        query_acceleration_max_scale_factor : int, optional
            Specifies the maximum scale factor for leasing compute resources for query acceleration. The scale factor is used as a multiplier based on warehouse size
        max_concurrency_level : int, optional
            Object parameter that specifies the concurrency level for SQL statements executed by a warehouse cluster
        statement_queued_timeout_in_seconds : int, optional
            Object parameter that specifies the time, in seconds, a SQL statement can be queued on a warehouse before it is canceled by the system
        statement_timeout_in_seconds : int, optional
            Object parameter that specifies the time, in seconds, after which a running SQL statement  is canceled by the system
        type : str, optional
            [Deprecated] Type of warehouse, possible types: STANDARD, SNOWPARK-OPTIMIZED
        size : str, optional
            [Deprecated] names of size: X-Small, Small, Medium, Large, X-Large, 2X-Large, 3X-Large, 4X-Large, 5X-Large, 6X-Large
        state : str, optional
            The state of warehouse, possible states: STARTED, STARTING, DYNAMIC, SUSPENDED, RESIZING, RESUMING, SUSPENDING
        started_clusters : int, optional
            Number of clusters currently started.
        running : int, optional
            Number of SQL statements that are being executed by the warehouse.
        queued : int, optional
            Number of SQL statements that are queued for the warehouse.
        is_default : bool, optional
            Whether the warehouse is the default for the current user.
        is_current : bool, optional
            Whether the warehouse is in use for the session. Only one warehouse can be in use at a time for a session.  To specify or change the warehouse for a session, use the USE WAREHOUSE command.
        available : str, optional
            Percentage of the warehouse compute resources that are provisioned and available.
        provisioning : str, optional
            Percentage of the warehouse compute resources that are in the process of provisioning.
        quiescing : str, optional
            Percentage of the warehouse compute resources that are executing SQL statements,  but will be shut down once the queries complete.
        other : str, optional
            Percentage of the warehouse compute resources that are in a state other than available,  provisioning, or quiescing.
        created_on : datetime, optional
            Date and time when the warehouse was created.
        resumed_on : datetime, optional
            Date and time when the warehouse was last started or restarted.
        updated_on : datetime, optional
            Date and time when the warehouse was last updated,  which includes changing any of the properties of the warehouse or changing the state (STARTED, SUSPENDED, RESIZING) of the warehouse.
        owner : str, optional
            Role that owns the warehouse.
        budget : str, optional
            Comment representing budget for warehouse.
        kind : str, optional

        owner_role_type : str, optional
            The type of role that owns the object.
        warehouse_credit_limit : int, optional
            Credit limit that are can be executed by the warehouse.
        target_statement_size : str, optional
            Names of size: X-Small, Small, Medium, Large, X-Large, 2X-Large, 3X-Large, 4X-Large, 5X-Large, 6X-Large
        """
        self.name = name
        self.warehouse_type = warehouse_type
        self.warehouse_size = warehouse_size
        self.wait_for_completion = wait_for_completion
        self.max_cluster_count = max_cluster_count
        self.min_cluster_count = min_cluster_count
        self.scaling_policy = scaling_policy
        self.auto_suspend = auto_suspend
        self.auto_resume = auto_resume
        self.initially_suspended = initially_suspended
        self.resource_monitor = resource_monitor
        self.comment = comment
        self.enable_query_acceleration = enable_query_acceleration
        self.query_acceleration_max_scale_factor = query_acceleration_max_scale_factor
        self.max_concurrency_level = max_concurrency_level
        self.statement_queued_timeout_in_seconds = statement_queued_timeout_in_seconds
        self.statement_timeout_in_seconds = statement_timeout_in_seconds
        self.type = type
        self.size = size
        self.state = state
        self.started_clusters = started_clusters
        self.running = running
        self.queued = queued
        self.is_default = is_default
        self.is_current = is_current
        self.available = available
        self.provisioning = provisioning
        self.quiescing = quiescing
        self.other = other
        self.created_on = created_on
        self.resumed_on = resumed_on
        self.updated_on = updated_on
        self.owner = owner
        self.budget = budget
        self.kind = kind
        self.owner_role_type = owner_role_type
        self.warehouse_credit_limit = warehouse_credit_limit
        self.target_statement_size = target_statement_size

    __properties = [
        "name",
        "warehouse_type",
        "warehouse_size",
        "wait_for_completion",
        "max_cluster_count",
        "min_cluster_count",
        "scaling_policy",
        "auto_suspend",
        "auto_resume",
        "initially_suspended",
        "resource_monitor",
        "comment",
        "enable_query_acceleration",
        "query_acceleration_max_scale_factor",
        "max_concurrency_level",
        "statement_queued_timeout_in_seconds",
        "statement_timeout_in_seconds",
        "type",
        "size",
        "state",
        "started_clusters",
        "running",
        "queued",
        "is_default",
        "is_current",
        "available",
        "provisioning",
        "quiescing",
        "other",
        "created_on",
        "resumed_on",
        "updated_on",
        "owner",
        "budget",
        "kind",
        "owner_role_type",
        "warehouse_credit_limit",
        "target_statement_size",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Warehouse(
            name=self.name,
            warehouse_type=self.warehouse_type,
            warehouse_size=self.warehouse_size,
            wait_for_completion=self.wait_for_completion,
            max_cluster_count=self.max_cluster_count,
            min_cluster_count=self.min_cluster_count,
            scaling_policy=self.scaling_policy,
            auto_suspend=self.auto_suspend,
            auto_resume=self.auto_resume,
            initially_suspended=self.initially_suspended,
            resource_monitor=self.resource_monitor,
            comment=self.comment,
            enable_query_acceleration=self.enable_query_acceleration,
            query_acceleration_max_scale_factor=self.query_acceleration_max_scale_factor,
            max_concurrency_level=self.max_concurrency_level,
            statement_queued_timeout_in_seconds=self.statement_queued_timeout_in_seconds,
            statement_timeout_in_seconds=self.statement_timeout_in_seconds,
            type=self.type,
            size=self.size,
            state=self.state,
            started_clusters=self.started_clusters,
            running=self.running,
            queued=self.queued,
            is_default=self.is_default,
            is_current=self.is_current,
            available=self.available,
            provisioning=self.provisioning,
            quiescing=self.quiescing,
            other=self.other,
            created_on=self.created_on,
            resumed_on=self.resumed_on,
            updated_on=self.updated_on,
            owner=self.owner,
            budget=self.budget,
            kind=self.kind,
            owner_role_type=self.owner_role_type,
            warehouse_credit_limit=self.warehouse_credit_limit,
            target_statement_size=self.target_statement_size,
        )

    @classmethod
    def _from_model(cls, model) -> WarehouseModel:
        return WarehouseModel(
            name=model.name,
            warehouse_type=model.warehouse_type,
            warehouse_size=model.warehouse_size,
            wait_for_completion=model.wait_for_completion,
            max_cluster_count=model.max_cluster_count,
            min_cluster_count=model.min_cluster_count,
            scaling_policy=model.scaling_policy,
            auto_suspend=model.auto_suspend,
            auto_resume=model.auto_resume,
            initially_suspended=model.initially_suspended,
            resource_monitor=model.resource_monitor,
            comment=model.comment,
            enable_query_acceleration=model.enable_query_acceleration,
            query_acceleration_max_scale_factor=model.query_acceleration_max_scale_factor,
            max_concurrency_level=model.max_concurrency_level,
            statement_queued_timeout_in_seconds=model.statement_queued_timeout_in_seconds,
            statement_timeout_in_seconds=model.statement_timeout_in_seconds,
            type=model.type,
            size=model.size,
            state=model.state,
            started_clusters=model.started_clusters,
            running=model.running,
            queued=model.queued,
            is_default=model.is_default,
            is_current=model.is_current,
            available=model.available,
            provisioning=model.provisioning,
            quiescing=model.quiescing,
            other=model.other,
            created_on=model.created_on,
            resumed_on=model.resumed_on,
            updated_on=model.updated_on,
            owner=model.owner,
            budget=model.budget,
            kind=model.kind,
            owner_role_type=model.owner_role_type,
            warehouse_credit_limit=model.warehouse_credit_limit,
            target_statement_size=model.target_statement_size,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Warehouse.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Warehouse object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> WarehouseModel:
        """Create an instance of Warehouse from a dict.

        This method constructs a Warehouse object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Warehouse
            A Warehouse object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Warehouse.from_dict(obj))


Warehouse._model_class = WarehouseModel



# coding: utf-8

# flake8: noqa

"""
Preview Feature Test API

Test spec for preview feature filtering  # noqa: E501

The version of the OpenAPI document: 1.0.0
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from tests.preview_test._generated.api.default_api import DefaultApi

# import models into sdk package
from tests.preview_test._generated.models.create_resource_request import CreateResourceRequest
from tests.preview_test._generated.models.resource import Resource
from tests.preview_test._generated.models.shared_model import SharedModel



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.account import Account, AccountCollection, AccountResource

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
ACCOUNT = Account("my_account", "STANDARD", "admin", "admin@localhost")


@pytest.fixture()
def accounts(fake_root):
    return AccountCollection(fake_root)


@pytest.fixture()
def account(accounts):
    return accounts["my_account"]


def test_create_async(fake_root, accounts):
    args = (fake_root, "POST", BASE_URL + "/accounts")
    kwargs = extra_params(
        body={
            "name": "my_account",
            "edition": "STANDARD",
            "admin_name": "admin",
            "email": "admin@localhost",
            "must_change_password": False,
            "polaris": False,
            "dropped_on": None,
            "scheduled_deletion_time": None,
            "restored_on": None,
            "organization_URL_expiration_on": None,
            "moved_on": None,
        }
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        account_res = accounts.create(ACCOUNT)
        assert isinstance(account_res, AccountResource)
        assert account_res.name == "my_account"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = accounts.create_async(ACCOUNT)
        assert isinstance(op, PollingOperation)
        account_res = op.result()
        assert account_res.name == "my_account"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_async(fake_root, accounts):
    args = (fake_root, "GET", BASE_URL + "/accounts")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = accounts.iter()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = accounts.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_async(fake_root, account):
    args = (fake_root, "DELETE", BASE_URL + "/accounts/my_account?gracePeriodInDays=7")
    kwargs = extra_params(query_params=[("gracePeriodInDays", 7)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        account.drop(7)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = account.drop_async(7)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_undrop_async(fake_root, account):
    args = (fake_root, "POST", BASE_URL + "/accounts/my_account:undrop")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        account.undrop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = account.undrop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



import argparse
import uuid

import snowflake.connector

from ..utils import (
    backup_database_and_schema,
    backup_role,
    backup_warehouse,
    connection_config,
    connection_keys,
    create_and_use_new_database_and_schema,
    create_zip_from_paths,
    execute_notebook,
    setup_account_for_notebook,
    upload_given_files_to_stage,
)


TEST_DATABASE_NAME = "DATABASE_PYTHON_TESTING_NOTEBOOK"
TEST_SCHEMA = "GH_JOB_{}".format(str(uuid.uuid4()).replace("-", "_"))
STAGE_NAME = "STAGE_PYTHON_TEST_NOTEBOOK"

all_notebook_files = [
    {
        "notebook_name": "python_test_notebook",
        "notebook_file_name": "python_test_notebook.ipynb",
        "have_output_file": True,
        "is_integration_test": True,
    },
    {"notebook_name": "doc_notebook_1", "notebook_file_name": "doc_notebook_1.ipynb", "have_output_file": False},
]

parser = argparse.ArgumentParser()
parser.add_argument("--tests-only", action="store_true", default=False, help="Only run the notebooks with tests")


def run(tests_only: bool):
    execution_output = None
    config = connection_config()
    should_fail_job = False
    if tests_only:
        notebooks_to_run = [notebook for notebook in all_notebook_files if notebook.get("is_integration_test")]
    else:
        notebooks_to_run = all_notebook_files

    # copy the snowpy package to the tests/integ/notebook folder
    snowflake_core_source_folder = "./src"
    snowflake_core_test_folder = "./tests"
    snowflake_core_pyproject_file_path = "./pyproject.toml"
    notebook_directory_path = "tests/integ/notebook"
    snowflake_core_zip_filename = "snowflake_core.zip"
    notebook_env_file_name = "environment.yml"
    notebook_execution_output_file_name = "execution_output.txt"
    snowflake_core_zip_filepath = f"{notebook_directory_path}/{snowflake_core_zip_filename}"

    # create the zip file
    create_zip_from_paths(
        [snowflake_core_source_folder, snowflake_core_test_folder, snowflake_core_pyproject_file_path],
        snowflake_core_zip_filepath,
    )

    _keys = connection_keys()

    with snowflake.connector.connect(
        # This works around SNOW-998521, by forcing JSON results
        **{k: config[k] for k in _keys if k in config}
    ) as connection:
        cursor = connection.cursor()

        # Set up account for notebook
        setup_account_for_notebook(cursor, config)

        with backup_database_and_schema(cursor), backup_warehouse(cursor), backup_role(cursor):
            try:
                create_and_use_new_database_and_schema(cursor, TEST_DATABASE_NAME, TEST_SCHEMA)
                warehouse_name = config["warehouse"]

                # create the stage and put the notebook files in the stage
                cursor.execute(f"CREATE STAGE {STAGE_NAME}")
                for notebook_file in notebooks_to_run:
                    stage_url = f"{STAGE_NAME}/{notebook_file['notebook_name']}"
                    files_to_upload = [
                        f"{notebook_directory_path}/{notebook_file['notebook_file_name']}",
                        f"{notebook_directory_path}/{notebook_env_file_name}",
                    ]
                    upload_given_files_to_stage(cursor, stage_url, files_to_upload)

                upload_given_files_to_stage(cursor, f"{STAGE_NAME}/zip", [snowflake_core_zip_filepath])

                # try to create and execute the notebook
                for notebook_file in notebooks_to_run:
                    print(f"Executing notebook {notebook_file['notebook_file_name']}")
                    should_fail_job_temp = execute_notebook(
                        cursor,
                        notebook_file["notebook_name"],
                        f"{TEST_DATABASE_NAME}.{TEST_SCHEMA}.{STAGE_NAME}/{notebook_file['notebook_name']}",
                        warehouse_name,
                        notebook_file["notebook_file_name"],
                    )
                    should_fail_job = should_fail_job or should_fail_job_temp
                    # fetch the output of the notebook execution and read
                    notebook_execution_output_file_name = f"{notebook_file['notebook_name']}_output.txt"
                    if notebook_file["have_output_file"]:
                        source_path = f"{TEST_DATABASE_NAME}.{TEST_SCHEMA}.{STAGE_NAME}/{notebook_file['notebook_name']}/{notebook_execution_output_file_name}"
                        target_directory = f"{notebook_directory_path}"
                        # download the output file
                        try:
                            cursor.execute(f"GET @{source_path} file://{target_directory}")
                        except Exception as e:
                            print(
                                f"Error downloading the notebook {notebook_file['notebook_file_name']}:\n {e.with_traceback(None)}"
                            )
                            should_fail_job = True
                            continue
                        # read the output file
                        try:
                            output_file_name = f"{target_directory}/{notebook_execution_output_file_name}"
                            with open(f"{output_file_name}") as f:
                                execution_output = f.read()
                                print(f"Notebook {notebook_file['notebook_name']} Output::::::STARTS")
                                print(execution_output)
                                print(f"Notebook {notebook_file['notebook_name']} Output::::::ENDS")
                        except Exception as e:
                            print(
                                f"Error reading the notebook {notebook_file['notebook_file_name']}:\n {e.with_traceback(None)}"
                            )
                            should_fail_job = True
                            continue
                        if notebook_file["is_integration_test"]:
                            if not (execution_output.find("All tests passed") != -1):
                                print("Some Integration test are failing")
                                should_fail_job = True
            finally:
                cursor.execute(f"USE DATABASE {TEST_DATABASE_NAME}")
                cursor.execute(f"USE SCHEMA {TEST_SCHEMA}")
                cursor.execute("DROP NOTEBOOK IF EXISTS python_test_notebook")
                cursor.execute(f"DROP STAGE IF EXISTS {STAGE_NAME}")
                cursor.execute(f"DROP SCHEMA IF EXISTS {TEST_SCHEMA}")
    if should_fail_job:
        raise Exception("Job Failed")


if __name__ == "__main__":
    args = parser.parse_args()
    tests_only: bool = args.tests_only
    run(tests_only)



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr, field_validator

from snowflake.core.cortex.analyst_service._generated.models.semantic_model_object import (
    SemanticModelObject,
    SemanticModelObjectModel,
)
from snowflake.core.cortex.analyst_service._generated.models.sql_inputs import SqlInputs, SqlInputsModel
from snowflake.core.cortex.analyst_service._generated.models.table_columns import TableColumns, TableColumnsModel
from snowflake.core.cortex.analyst_service._generated.models.tds_input import TdsInput, TdsInputModel


class GenerateVerifiedQuerySuggestionsRequest(BaseModel):
    """A model object representing the GenerateVerifiedQuerySuggestionsRequest resource.

    Constructs an object of type GenerateVerifiedQuerySuggestionsRequest with the provided properties.

    Parameters
    __________
    warehouse : str
        Warehouse name to use for processing suggestions.
    mode : str,  default 'ca_requests_based'
        Whether to generate suggestions based on historical CA requests or all SQL queries. One of 'ca_requests_based' or 'query_history_based'
    semantic_model : SemanticModelObject, optional

    tables : list[TableColumns], optional

    tds_source : list[TdsInput], optional

    sql_source : SqlInputs, optional

    offset : int, optional
        The suggestions offset index, offset = 0 indicates the first call
    limit : int, optional
        The number of suggestions to return starting from the provided offset value
    experimental : str, optional
        JSON serialized string of experimental API fields (undocumented).
    """

    mode: Optional[StrictStr] = "ca_requests_based"

    semantic_model: Optional[SemanticModelObject] = None

    tables: Optional[List[TableColumns]] = None

    tds_source: Optional[List[TdsInput]] = None

    sql_source: Optional[SqlInputs] = None

    warehouse: StrictStr

    offset: Optional[StrictInt] = None

    limit: Optional[StrictInt] = None

    experimental: Optional[StrictStr] = None

    __properties = [
        "mode",
        "semantic_model",
        "tables",
        "tds_source",
        "sql_source",
        "warehouse",
        "offset",
        "limit",
        "experimental",
    ]

    @field_validator("mode")
    def mode_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("ca_requests_based", "query_history_based"):
            raise ValueError("must validate the enum values ('ca_requests_based','query_history_based')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GenerateVerifiedQuerySuggestionsRequest:
        """Create an instance of GenerateVerifiedQuerySuggestionsRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of semantic_model
        if self.semantic_model:
            _dict["semantic_model"] = self.semantic_model.to_dict()

        # override the default output from pydantic by calling `to_dict()` of each item in tables (list)
        _items = []
        if self.tables:
            for _item in self.tables:
                if _item:
                    _items.append(_item.to_dict())
            _dict["tables"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in tds_source (list)
        _items = []
        if self.tds_source:
            for _item in self.tds_source:
                if _item:
                    _items.append(_item.to_dict())
            _dict["tds_source"] = _items

        # override the default output from pydantic by calling `to_dict()` of sql_source
        if self.sql_source:
            _dict["sql_source"] = self.sql_source.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> GenerateVerifiedQuerySuggestionsRequest:
        """Create an instance of GenerateVerifiedQuerySuggestionsRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return GenerateVerifiedQuerySuggestionsRequest.model_validate(obj)

        _obj = GenerateVerifiedQuerySuggestionsRequest.model_validate(
            {
                "mode": obj.get("mode") if obj.get("mode") is not None else "ca_requests_based",
                "semantic_model": SemanticModelObject.from_dict(obj.get("semantic_model"))
                if obj.get("semantic_model") is not None
                else None,
                "tables": [TableColumns.from_dict(_item) for _item in obj.get("tables")]
                if obj.get("tables") is not None
                else None,
                "tds_source": [TdsInput.from_dict(_item) for _item in obj.get("tds_source")]
                if obj.get("tds_source") is not None
                else None,
                "sql_source": SqlInputs.from_dict(obj.get("sql_source")) if obj.get("sql_source") is not None else None,
                "warehouse": obj.get("warehouse"),
                "offset": obj.get("offset"),
                "limit": obj.get("limit"),
                "experimental": obj.get("experimental"),
            }
        )

        return _obj


class GenerateVerifiedQuerySuggestionsRequestModel:
    def __init__(
        self,
        warehouse: str,
        # optional properties
        mode: Optional[str] = "ca_requests_based",
        semantic_model: Optional[SemanticModelObject] = None,
        tables: Optional[list[TableColumns]] = None,
        tds_source: Optional[list[TdsInput]] = None,
        sql_source: Optional[SqlInputs] = None,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        experimental: Optional[str] = None,
    ):
        """A model object representing the GenerateVerifiedQuerySuggestionsRequest resource.

        Constructs an object of type GenerateVerifiedQuerySuggestionsRequest with the provided properties.

        Parameters
        __________
        warehouse : str
            Warehouse name to use for processing suggestions.
        mode : str,  default 'ca_requests_based'
            Whether to generate suggestions based on historical CA requests or all SQL queries. One of 'ca_requests_based' or 'query_history_based'
        semantic_model : SemanticModelObject, optional

        tables : list[TableColumns], optional

        tds_source : list[TdsInput], optional

        sql_source : SqlInputs, optional

        offset : int, optional
            The suggestions offset index, offset = 0 indicates the first call
        limit : int, optional
            The number of suggestions to return starting from the provided offset value
        experimental : str, optional
            JSON serialized string of experimental API fields (undocumented).
        """
        self.mode = mode
        self.semantic_model = semantic_model
        self.tables = tables
        self.tds_source = tds_source
        self.sql_source = sql_source
        self.warehouse = warehouse
        self.offset = offset
        self.limit = limit
        self.experimental = experimental

    __properties = [
        "mode",
        "semantic_model",
        "tables",
        "tds_source",
        "sql_source",
        "warehouse",
        "offset",
        "limit",
        "experimental",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return GenerateVerifiedQuerySuggestionsRequest(
            mode=self.mode,
            semantic_model=self.semantic_model._to_model() if self.semantic_model is not None else None,
            tables=[x._to_model() for x in self.tables] if self.tables is not None else None,
            tds_source=[x._to_model() for x in self.tds_source] if self.tds_source is not None else None,
            sql_source=self.sql_source._to_model() if self.sql_source is not None else None,
            warehouse=self.warehouse,
            offset=self.offset,
            limit=self.limit,
            experimental=self.experimental,
        )

    @classmethod
    def _from_model(cls, model) -> GenerateVerifiedQuerySuggestionsRequestModel:
        return GenerateVerifiedQuerySuggestionsRequestModel(
            mode=model.mode,
            semantic_model=SemanticModelObjectModel._from_model(model.semantic_model) if model.semantic_model else None,
            tables=[TableColumnsModel._from_model(x) for x in model.tables] if model.tables else None,
            tds_source=[TdsInputModel._from_model(x) for x in model.tds_source] if model.tds_source else None,
            sql_source=SqlInputsModel._from_model(model.sql_source) if model.sql_source else None,
            warehouse=model.warehouse,
            offset=model.offset,
            limit=model.limit,
            experimental=model.experimental,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a GenerateVerifiedQuerySuggestionsRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the GenerateVerifiedQuerySuggestionsRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> GenerateVerifiedQuerySuggestionsRequestModel:
        """Create an instance of GenerateVerifiedQuerySuggestionsRequest from a dict.

        This method constructs a GenerateVerifiedQuerySuggestionsRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        GenerateVerifiedQuerySuggestionsRequest
            A GenerateVerifiedQuerySuggestionsRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(GenerateVerifiedQuerySuggestionsRequest.from_dict(obj))


GenerateVerifiedQuerySuggestionsRequest._model_class = GenerateVerifiedQuerySuggestionsRequestModel



from typing import TYPE_CHECKING, Optional

from pydantic import StrictStr

from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core.session._generated.api import SessionApi


if TYPE_CHECKING:
    from snowflake.core import Root


class SnowAPISession:
    def __init__(self, root: "Root") -> None:
        self.root = root
        self._api = SessionApi(
            root=root,
            # There is no Session resource that we are exposing publicly, yet.
            resource_class=None,
            sproc_client=StoredProcApiClient(root=self.root),
        )

    def _get_api_enablement_parameters(self) -> dict[str, str]:
        return self.get_parameters(filter_str="ENABLE_SNOW_API_FOR_%")

    def get_parameters(self, filter_str: Optional[StrictStr]) -> dict[str, str]:
        params = self._api.get_parameters(like=filter_str)

        params_map = dict()

        for param in params:
            p = param.to_dict()
            params_map[p["name"].lower()] = p["value"].lower()

        return params_map



"""
Snowflake Table API.

The Snowflake Table API is a REST API that you can use to access, update, and perform certain actions on Tables resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr, field_validator

from snowflake.core.table._generated.models.constraint import Constraint, ConstraintModel
from snowflake.core.table._generated.models.table_column import TableColumn, TableColumnModel


class Table(BaseModel):
    """A model object representing the Table resource.

    Constructs an object of type Table with the provided properties.

    Parameters
    __________
    name : str
        Specifies the name for the table, must be unique for the schema in which the table is created
    kind : str,  default 'PERMANENT'
        Table type - permanent, transient, or temporary
    cluster_by : list[str], optional
        Specifies one or more columns or column expressions in the table as the clustering key
    enable_schema_evolution : bool, optional
        Table has schema evolution enabled or disabled
    change_tracking : bool, optional
        Change tracking is enabled or disabled
    data_retention_time_in_days : int, optional
        Specifies the retention period for the table so that Time Travel actions SELECT, CLONE, UNDROP can be performed on historical data in the table
    max_data_extension_time_in_days : int, optional
        Specifies the retention period for the table so that Time Travel actions SELECT, CLONE, UNDROP can be performed on historical data in the table
    default_ddl_collation : str, optional
        Specifies a default collation specification for the columns in the table, including columns added to the table in the future
    columns : list[TableColumn], optional

    constraints : list[Constraint], optional

    comment : str, optional
        Comment for the table
    created_on : datetime, optional
        Date and time when the table was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the table is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the table is stored — **Read-only:** *any user-provided value will be ignored.*
    rows : int, optional
        Number of rows in the table. Returns NULL for external tables — **Read-only:** *any user-provided value will be ignored.*
    bytes : int, optional
        Number of bytes that will be scanned if the entire table is scanned in a query. Note that this number may be different than the number of actual physical bytes stored on-disk for the table — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the table — **Read-only:** *any user-provided value will be ignored.*
    dropped_on : datetime, optional
        Date and time when the table was dropped — **Read-only:** *any user-provided value will be ignored.*
    automatic_clustering : bool, optional
        If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the table — **Read-only:** *any user-provided value will be ignored.*
    search_optimization : bool, optional
        If ON, the table has the search optimization service enabled — **Read-only:** *any user-provided value will be ignored.*
    search_optimization_progress : int, optional
        Percentage of the table that has been optimized for search — **Read-only:** *any user-provided value will be ignored.*
    search_optimization_bytes : int, optional
        Number of additional bytes of storage that the search optimization service consumes for this table — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the object — **Read-only:** *any user-provided value will be ignored.*
    budget : str, optional
        Name of the budget if the object is monitored by a budget — **Read-only:** *any user-provided value will be ignored.*
    table_type : str, optional
        Type of the table — **Read-only:** *any user-provided value will be ignored.*
    """

    name: StrictStr

    kind: Optional[StrictStr] = "PERMANENT"

    cluster_by: Optional[List[StrictStr]] = None

    enable_schema_evolution: Optional[StrictBool] = None

    change_tracking: Optional[StrictBool] = None

    data_retention_time_in_days: Optional[StrictInt] = None

    max_data_extension_time_in_days: Optional[StrictInt] = None

    default_ddl_collation: Optional[StrictStr] = None

    columns: Optional[List[TableColumn]] = None

    constraints: Optional[List[Constraint]] = None

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    rows: Optional[StrictInt] = None

    bytes: Optional[StrictInt] = None

    owner: Optional[StrictStr] = None

    dropped_on: Optional[datetime] = None

    automatic_clustering: Optional[StrictBool] = None

    search_optimization: Optional[StrictBool] = None

    search_optimization_progress: Optional[StrictInt] = None

    search_optimization_bytes: Optional[StrictInt] = None

    owner_role_type: Optional[StrictStr] = None

    budget: Optional[StrictStr] = None

    table_type: Optional[StrictStr] = None

    __properties = [
        "name",
        "kind",
        "cluster_by",
        "enable_schema_evolution",
        "change_tracking",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "default_ddl_collation",
        "columns",
        "constraints",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "rows",
        "bytes",
        "owner",
        "dropped_on",
        "automatic_clustering",
        "search_optimization",
        "search_optimization_progress",
        "search_optimization_bytes",
        "owner_role_type",
        "budget",
        "table_type",
    ]

    @field_validator("kind")
    def kind_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("PERMANENT", "TRANSIENT", "TEMPORARY", "", "transient", "temporary"):
            raise ValueError(
                "must validate the enum values ('PERMANENT','TRANSIENT','TEMPORARY','','transient','temporary')"
            )
        return v

    @field_validator("table_type")
    def table_type_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("NORMAL", "DYNAMIC", "EXTERNAL", "EVENT", "HYBRID", "ICEBERG", "IMMUTABLE"):
            raise ValueError(
                "must validate the enum values ('NORMAL','DYNAMIC','EXTERNAL','EVENT','HYBRID','ICEBERG','IMMUTABLE')"
            )
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Table:
        """Create an instance of Table from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "rows",
                    "bytes",
                    "owner",
                    "dropped_on",
                    "automatic_clustering",
                    "search_optimization",
                    "search_optimization_progress",
                    "search_optimization_bytes",
                    "owner_role_type",
                    "budget",
                    "table_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict["columns"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in constraints (list)
        _items = []
        if self.constraints:
            for _item in self.constraints:
                if _item:
                    _items.append(_item.to_dict())
            _dict["constraints"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Table:
        """Create an instance of Table from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Table.model_validate(obj)

        _obj = Table.model_validate(
            {
                "name": obj.get("name"),
                "kind": obj.get("kind") if obj.get("kind") is not None else "PERMANENT",
                "cluster_by": obj.get("cluster_by"),
                "enable_schema_evolution": obj.get("enable_schema_evolution"),
                "change_tracking": obj.get("change_tracking"),
                "data_retention_time_in_days": obj.get("data_retention_time_in_days"),
                "max_data_extension_time_in_days": obj.get("max_data_extension_time_in_days"),
                "default_ddl_collation": obj.get("default_ddl_collation"),
                "columns": [TableColumn.from_dict(_item) for _item in obj.get("columns")]
                if obj.get("columns") is not None
                else None,
                "constraints": [Constraint.from_dict(_item) for _item in obj.get("constraints")]
                if obj.get("constraints") is not None
                else None,
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "rows": obj.get("rows"),
                "bytes": obj.get("bytes"),
                "owner": obj.get("owner"),
                "dropped_on": obj.get("dropped_on"),
                "automatic_clustering": obj.get("automatic_clustering"),
                "search_optimization": obj.get("search_optimization"),
                "search_optimization_progress": obj.get("search_optimization_progress"),
                "search_optimization_bytes": obj.get("search_optimization_bytes"),
                "owner_role_type": obj.get("owner_role_type"),
                "budget": obj.get("budget"),
                "table_type": obj.get("table_type"),
            }
        )

        return _obj


class TableModel:
    def __init__(
        self,
        name: str,
        # optional properties
        kind: Optional[str] = "PERMANENT",
        cluster_by: Optional[list[str]] = None,
        enable_schema_evolution: Optional[bool] = None,
        change_tracking: Optional[bool] = None,
        data_retention_time_in_days: Optional[int] = None,
        max_data_extension_time_in_days: Optional[int] = None,
        default_ddl_collation: Optional[str] = None,
        columns: Optional[list[TableColumn]] = None,
        constraints: Optional[list[Constraint]] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        rows: Optional[int] = None,
        bytes: Optional[int] = None,
        owner: Optional[str] = None,
        dropped_on: Optional[datetime] = None,
        automatic_clustering: Optional[bool] = None,
        search_optimization: Optional[bool] = None,
        search_optimization_progress: Optional[int] = None,
        search_optimization_bytes: Optional[int] = None,
        owner_role_type: Optional[str] = None,
        budget: Optional[str] = None,
        table_type: Optional[str] = None,
    ):
        """A model object representing the Table resource.

        Constructs an object of type Table with the provided properties.

        Parameters
        __________
        name : str
            Specifies the name for the table, must be unique for the schema in which the table is created
        kind : str,  default 'PERMANENT'
            Table type - permanent, transient, or temporary
        cluster_by : list[str], optional
            Specifies one or more columns or column expressions in the table as the clustering key
        enable_schema_evolution : bool, optional
            Table has schema evolution enabled or disabled
        change_tracking : bool, optional
            Change tracking is enabled or disabled
        data_retention_time_in_days : int, optional
            Specifies the retention period for the table so that Time Travel actions SELECT, CLONE, UNDROP can be performed on historical data in the table
        max_data_extension_time_in_days : int, optional
            Specifies the retention period for the table so that Time Travel actions SELECT, CLONE, UNDROP can be performed on historical data in the table
        default_ddl_collation : str, optional
            Specifies a default collation specification for the columns in the table, including columns added to the table in the future
        columns : list[TableColumn], optional

        constraints : list[Constraint], optional

        comment : str, optional
            Comment for the table
        created_on : datetime, optional
            Date and time when the table was created.
        database_name : str, optional
            Database in which the table is stored
        schema_name : str, optional
            Schema in which the table is stored
        rows : int, optional
            Number of rows in the table. Returns NULL for external tables.
        bytes : int, optional
            Number of bytes that will be scanned if the entire table is scanned in a query. Note that this number may be different than the number of actual physical bytes stored on-disk for the table
        owner : str, optional
            Role that owns the table
        dropped_on : datetime, optional
            Date and time when the table was dropped
        automatic_clustering : bool, optional
            If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the table.
        search_optimization : bool, optional
            If ON, the table has the search optimization service enabled
        search_optimization_progress : int, optional
            Percentage of the table that has been optimized for search.
        search_optimization_bytes : int, optional
            Number of additional bytes of storage that the search optimization service consumes for this table
        owner_role_type : str, optional
            The type of role that owns the object.
        budget : str, optional
            Name of the budget if the object is monitored by a budget
        table_type : str, optional
            Type of the table
        """
        self.name = name
        self.kind = kind
        self.cluster_by = cluster_by
        self.enable_schema_evolution = enable_schema_evolution
        self.change_tracking = change_tracking
        self.data_retention_time_in_days = data_retention_time_in_days
        self.max_data_extension_time_in_days = max_data_extension_time_in_days
        self.default_ddl_collation = default_ddl_collation
        self.columns = columns
        self.constraints = constraints
        self.comment = comment
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.rows = rows
        self.bytes = bytes
        self.owner = owner
        self.dropped_on = dropped_on
        self.automatic_clustering = automatic_clustering
        self.search_optimization = search_optimization
        self.search_optimization_progress = search_optimization_progress
        self.search_optimization_bytes = search_optimization_bytes
        self.owner_role_type = owner_role_type
        self.budget = budget
        self.table_type = table_type

    __properties = [
        "name",
        "kind",
        "cluster_by",
        "enable_schema_evolution",
        "change_tracking",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "default_ddl_collation",
        "columns",
        "constraints",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "rows",
        "bytes",
        "owner",
        "dropped_on",
        "automatic_clustering",
        "search_optimization",
        "search_optimization_progress",
        "search_optimization_bytes",
        "owner_role_type",
        "budget",
        "table_type",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Table(
            name=self.name,
            kind=self.kind,
            cluster_by=self.cluster_by,
            enable_schema_evolution=self.enable_schema_evolution,
            change_tracking=self.change_tracking,
            data_retention_time_in_days=self.data_retention_time_in_days,
            max_data_extension_time_in_days=self.max_data_extension_time_in_days,
            default_ddl_collation=self.default_ddl_collation,
            columns=[x._to_model() for x in self.columns] if self.columns is not None else None,
            constraints=[x._to_model() for x in self.constraints] if self.constraints is not None else None,
            comment=self.comment,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            rows=self.rows,
            bytes=self.bytes,
            owner=self.owner,
            dropped_on=self.dropped_on,
            automatic_clustering=self.automatic_clustering,
            search_optimization=self.search_optimization,
            search_optimization_progress=self.search_optimization_progress,
            search_optimization_bytes=self.search_optimization_bytes,
            owner_role_type=self.owner_role_type,
            budget=self.budget,
            table_type=self.table_type,
        )

    @classmethod
    def _from_model(cls, model) -> TableModel:
        return TableModel(
            name=model.name,
            kind=model.kind,
            cluster_by=model.cluster_by,
            enable_schema_evolution=model.enable_schema_evolution,
            change_tracking=model.change_tracking,
            data_retention_time_in_days=model.data_retention_time_in_days,
            max_data_extension_time_in_days=model.max_data_extension_time_in_days,
            default_ddl_collation=model.default_ddl_collation,
            columns=[TableColumnModel._from_model(x) for x in model.columns] if model.columns else None,
            constraints=[ConstraintModel._from_model(x) for x in model.constraints] if model.constraints else None,
            comment=model.comment,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            rows=model.rows,
            bytes=model.bytes,
            owner=model.owner,
            dropped_on=model.dropped_on,
            automatic_clustering=model.automatic_clustering,
            search_optimization=model.search_optimization,
            search_optimization_progress=model.search_optimization_progress,
            search_optimization_bytes=model.search_optimization_bytes,
            owner_role_type=model.owner_role_type,
            budget=model.budget,
            table_type=model.table_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Table.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Table object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableModel:
        """Create an instance of Table from a dict.

        This method constructs a Table object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Table
            A Table object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Table.from_dict(obj))


Table._model_class = TableModel



"""
Snowflake External Volume API.

The Snowflake External Volume API is a REST API that you can use to access, update, and perform certain actions on External Volume resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.external_volume._generated.models.storage_location import StorageLocation, StorageLocationModel


class ExternalVolume(BaseModel):
    """A model object representing the ExternalVolume resource.

    Constructs an object of type ExternalVolume with the provided properties.

    Parameters
    __________
    name : str
        String that specifies the identifier (the name) for the external volume; must be unique in your account.
    storage_locations : list[StorageLocation]
        Set of named cloud storage locations in different regions and, optionally, cloud platforms.
    allow_writes : bool, optional
        Specifies whether write operations are allowed for the external volume; must be set to TRUE for Iceberg tables that use Snowflake as the catalog.
    comment : str, optional
        String (literal) that specifies a comment for the external volume.
    created_on : datetime, optional
        Date and time when the external volume was created — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the external volume — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the external volume — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    storage_locations: List[StorageLocation]

    allow_writes: Optional[StrictBool] = None

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    owner: Optional[Annotated[str, Field(strict=True)]] = None

    owner_role_type: Optional[Annotated[str, Field(strict=True)]] = None

    __properties = ["name", "storage_locations", "allow_writes", "comment", "created_on", "owner", "owner_role_type"]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("owner")
    def owner_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("owner_role_type")
    def owner_role_type_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ExternalVolume:
        """Create an instance of ExternalVolume from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "owner",
                    "owner_role_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in storage_locations (list)
        _items = []
        if self.storage_locations:
            for _item in self.storage_locations:
                if _item:
                    _items.append(_item.to_dict())
            _dict["storage_locations"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ExternalVolume:
        """Create an instance of ExternalVolume from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ExternalVolume.model_validate(obj)

        _obj = ExternalVolume.model_validate(
            {
                "name": obj.get("name"),
                "storage_locations": [StorageLocation.from_dict(_item) for _item in obj.get("storage_locations")]
                if obj.get("storage_locations") is not None
                else None,
                "allow_writes": obj.get("allow_writes"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
            }
        )

        return _obj


class ExternalVolumeModel:
    def __init__(
        self,
        name: str,
        storage_locations: list[StorageLocation],
        # optional properties
        allow_writes: Optional[bool] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
    ):
        """A model object representing the ExternalVolume resource.

        Constructs an object of type ExternalVolume with the provided properties.

        Parameters
        __________
        name : str
            String that specifies the identifier (the name) for the external volume; must be unique in your account.
        storage_locations : list[StorageLocation]
            Set of named cloud storage locations in different regions and, optionally, cloud platforms.
        allow_writes : bool, optional
            Specifies whether write operations are allowed for the external volume; must be set to TRUE for Iceberg tables that use Snowflake as the catalog.
        comment : str, optional
            String (literal) that specifies a comment for the external volume.
        created_on : datetime, optional
            Date and time when the external volume was created.
        owner : str, optional
            Role that owns the external volume
        owner_role_type : str, optional
            The type of role that owns the external volume
        """
        self.name = name
        self.storage_locations = storage_locations
        self.allow_writes = allow_writes
        self.comment = comment
        self.created_on = created_on
        self.owner = owner
        self.owner_role_type = owner_role_type

    __properties = ["name", "storage_locations", "allow_writes", "comment", "created_on", "owner", "owner_role_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ExternalVolume(
            name=self.name,
            storage_locations=[x._to_model() for x in self.storage_locations]
            if self.storage_locations is not None
            else None,
            allow_writes=self.allow_writes,
            comment=self.comment,
            created_on=self.created_on,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
        )

    @classmethod
    def _from_model(cls, model) -> ExternalVolumeModel:
        return ExternalVolumeModel(
            name=model.name,
            storage_locations=[StorageLocationModel._from_model(x) for x in model.storage_locations]
            if model.storage_locations
            else None,
            allow_writes=model.allow_writes,
            comment=model.comment,
            created_on=model.created_on,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ExternalVolume.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ExternalVolume object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ExternalVolumeModel:
        """Create an instance of ExternalVolume from a dict.

        This method constructs a ExternalVolume object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ExternalVolume
            A ExternalVolume object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ExternalVolume.from_dict(obj))


ExternalVolume._model_class = ExternalVolumeModel



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from snowflake.core import PollingOperation
from snowflake.core._common import CreateMode
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._internal.utils import deprecated
from snowflake.core.compute_pool._generated.models.compute_pool import ComputePoolModel as ComputePool

from ._generated.api.compute_pool_api_base import ComputePoolCollectionBase, ComputePoolResourceBase


if TYPE_CHECKING:
    from snowflake.core import Root


class ComputePoolCollection(ComputePoolCollectionBase):
    """Represents the collection operations on the Snowflake Compute Pool resource.

    With this collection, you can create, iterate through, and search for compute pools that you have access to
    in the current context.

    Examples
    ________
    Creating a compute pool instance:

    >>> compute_pool = ComputePool(
    ...     name="my_compute_pool", instance_family="CPU_X64_XS", min_nodes=1, max_nodes=2
    ... )
    >>> compute_pool_reference = root.compute_pools.create(compute_pool)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, ComputePoolResource)

    @api_telemetry
    def create(  # type: ignore[override]
        self,
        compute_pool: ComputePool,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
        initially_suspended: bool = False,
    ) -> "ComputePoolResource":
        """Create a compute pool in Snowflake.

        Parameters
        __________
        compute_pool: ComputePool
            The ``ComputePool`` object, together with the ``ComputePool``'s properties:
            name, min_nodes, max_nodes, instance_family;
            auto_resume, initially_suspended, auto_suspend_secs, comment are optional
        mode: CreateMode, optional
            One of the below enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError` if the compute
            pool already exists in Snowflake. Equivalent to SQL ``create compute pool <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the compute pool already exists in Snowflake. Equivalent to SQL
            ``create compute pool <name> if not exists...``.

            Default value is ``CreateMode.error_if_exists``.
        initially_suspended: bool, optional
            Determines if the compute pool should be suspended when initially created. Default value is False.

        Examples
        ________
        Creating a compute pool, replacing an existing compute pool with the same name:

        >>> compute_pool = ComputePool(
        ...     name="my_compute_pool", instance_family="CPU_X64_XS", min_nodes=1, max_nodes=2
        ... )
        >>> compute_pool_reference = compute_pools.create(compute_pool, mode=CreateMode.or_replace)

        Creating a compute pool that is initially suspended:

        >>> compute_pool = ComputePool(
        ...     name="my_compute_pool", instance_family="CPU_X64_XS", min_nodes=1, max_nodes=5
        ... )
        >>> compute_pool_reference = compute_pools.create(compute_pool, initially_suspended=True)
        """
        super().create(
            compute_pool=compute_pool._to_model(),
            mode=mode,
            initially_suspended=initially_suspended,
        )
        return self[compute_pool.name]

    @api_telemetry
    def create_async(  # type: ignore[override]
        self,
        compute_pool: ComputePool,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
        initially_suspended: bool = False,
    ) -> PollingOperation["ComputePoolResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().create_async(
            compute_pool=compute_pool._to_model(),
            mode=mode,
            initially_suspended=initially_suspended,
        )
        return PollingOperation(future, lambda _: self[compute_pool.name])

    @api_telemetry
    def iter(  # type: ignore[override]
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
    ) -> Iterator[ComputePool]:
        """Iterate through ``Compute Pool`` objects in Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear
            at the beginning of the object name. Uses case-sensitive pattern matching.
        limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.

        Examples
        ________
        Showing all compute pools that you have access to see:

        >>> compute_pools = root.compute_pools.iter()

        Showing information of the exact compute pool you want to see:

        >>> compute_pools = root.compute_pools.iter(like="your-compute-pool-name")

        Showing compute pools starting with 'your-compute-pool-name-':

        >>> compute_pools = root.compute_pools.iter(like="your-compute-pool-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for compute_pool in compute_pools:
        >>>     print(compute_pool.name)
        """
        compute_pools = super().iter(
            like=like,
            starts_with=starts_with,
            limit=limit,
        )
        return map(ComputePool._from_model, iter(compute_pools))

    @api_telemetry
    def iter_async(  # type: ignore[override]
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
    ) -> PollingOperation[Iterator[ComputePool]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().iter_async(
            like=like,
            starts_with=starts_with,
            limit=limit,
        )
        return PollingOperation(future, lambda rest_models: map(ComputePool._from_model, iter(rest_models)))


class ComputePoolResource(ComputePoolResourceBase):
    """Represents a reference to a Snowflake compute pool.

    With this compute pool reference, you can fetch information about a compute pool, as well as
    perform certain actions on it.
    """

    _plural_name = "compute_pools"

    @api_telemetry
    def create_or_alter(self, compute_pool: ComputePool) -> None:  # type: ignore[override]
        """Create a compute pool in Snowflake or alter one if it already exists.

        Parameters
        __________
        compute_pool: ComputePool
            An instance of :class:`ComputePool`.

        Examples
        ________
        Creating or updating a compute pool in Snowflake:

        >>> cp_parameters = ComputePool(
        ...     name="your-cp-name",
        ...     instance_family="CPU_X64_XS",
        ...     min_nodes=1,
        ...     max_nodes=1,
        ...)

        # Using a ``ComputePoolCollection`` to create or update a compute pool in Snowflake:
        >>> root.compute_pools["your-cp-name"].create_or_alter(cp_parameters)
        """
        super().create_or_alter(compute_pool._to_model())

    @api_telemetry
    def create_or_alter_async(self, compute_pool: ComputePool) -> PollingOperation[None]:  # type: ignore[override]
        """An asynchronous version of :func:`create_or_alter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return super().create_or_alter_async(compute_pool._to_model())

    @api_telemetry
    @deprecated("drop")
    def delete(self) -> None:
        self.drop()

    @api_telemetry
    def fetch(  # type: ignore[override]
        self,
    ) -> ComputePool:
        """Fetch a compute pool.

        Parameters
        __________
        """
        return ComputePool(**super().fetch().to_dict())

    @api_telemetry
    def fetch_async(  # type: ignore[override]
        self,
    ) -> PollingOperation[ComputePool]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_compute_pool(
            self.name,
            async_req=True,
        )

        return PollingOperation(future, lambda rest_model: ComputePool(**rest_model.to_dict()))



"""
Snowflake Managed Account API.

The Snowflake Managed Account API is a REST API that you can use to access, update, and perform certain actions on Managed Account resource in Snowflake.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, SecretStr, StrictStr, field_validator
from typing_extensions import Annotated


class ManagedAccount(BaseModel):
    """A model object representing the ManagedAccount resource.

    Constructs an object of type ManagedAccount with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    admin_name : str
        Name of the account administrator.
    admin_password : str
        Password for the account administrator.
    account_type : str
        Type of the account.
    cloud : str, optional
        Cloud in which the managed account is located. For reader accounts, this is always the same as the cloud for the provider account — **Read-only:** *any user-provided value will be ignored.*
    region : str, optional
        Region in which the managed account is located. For reader accounts, this is always the same as the region for the provider account — **Read-only:** *any user-provided value will be ignored.*
    locator : str, optional
        Legacy identifier for the account — **Read-only:** *any user-provided value will be ignored.*
    created_on : datetime, optional
        Date and time the account was created — **Read-only:** *any user-provided value will be ignored.*
    url : str, optional
        Account URL that is used to connect to the account, in the account name format. The account identifier in this format follows the pattern <orgname>-<account_name> — **Read-only:** *any user-provided value will be ignored.*
    account_locator_url : str, optional
        Account URL that is used to connect to the account, in the legacy account locator format — **Read-only:** *any user-provided value will be ignored.*
    comment : str, optional
        Optional comment in which to store information related to the account.
    """

    name: Annotated[str, Field(strict=True)]

    cloud: Optional[StrictStr] = None

    region: Optional[StrictStr] = None

    locator: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    url: Optional[StrictStr] = None

    account_locator_url: Optional[StrictStr] = None

    comment: Optional[StrictStr] = None

    admin_name: Optional[StrictStr]

    admin_password: Optional[SecretStr]

    account_type: StrictStr

    __properties = [
        "name",
        "cloud",
        "region",
        "locator",
        "created_on",
        "url",
        "account_locator_url",
        "comment",
        "admin_name",
        "admin_password",
        "account_type",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("account_type")
    def account_type_validate_enum(cls, v):
        if v not in ("READER"):
            raise ValueError("must validate the enum values ('READER')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ManagedAccount:
        """Create an instance of ManagedAccount from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "cloud",
                    "region",
                    "locator",
                    "created_on",
                    "url",
                    "account_locator_url",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if admin_name (nullable) is None
        if self.admin_name is None:
            _dict["admin_name"] = None

        # set to None if admin_password (nullable) is None
        if self.admin_password is None:
            _dict["admin_password"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ManagedAccount:
        """Create an instance of ManagedAccount from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ManagedAccount.model_validate(obj)

        _obj = ManagedAccount.model_validate(
            {
                "name": obj.get("name"),
                "cloud": obj.get("cloud"),
                "region": obj.get("region"),
                "locator": obj.get("locator"),
                "created_on": obj.get("created_on"),
                "url": obj.get("url"),
                "account_locator_url": obj.get("account_locator_url"),
                "comment": obj.get("comment"),
                "admin_name": obj.get("admin_name"),
                "admin_password": obj.get("admin_password"),
                "account_type": obj.get("account_type") if obj.get("account_type") is not None else "READER",
            }
        )

        return _obj


class ManagedAccountModel:
    def __init__(
        self,
        name: str,
        admin_name: str,
        admin_password: str,
        account_type: str = "READER",
        # optional properties
        cloud: Optional[str] = None,
        region: Optional[str] = None,
        locator: Optional[str] = None,
        created_on: Optional[datetime] = None,
        url: Optional[str] = None,
        account_locator_url: Optional[str] = None,
        comment: Optional[str] = None,
    ):
        """A model object representing the ManagedAccount resource.

        Constructs an object of type ManagedAccount with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        admin_name : str
            Name of the account administrator.
        admin_password : str
            Password for the account administrator.
        account_type : str
            Type of the account.
        cloud : str, optional
            Cloud in which the managed account is located. For reader accounts, this is always the same as the cloud for the provider account.
        region : str, optional
            Region in which the managed account is located. For reader accounts, this is always the same as the region for the provider account.
        locator : str, optional
            Legacy identifier for the account.
        created_on : datetime, optional
            Date and time the account was created.
        url : str, optional
            Account URL that is used to connect to the account, in the account name format. The account identifier in this format follows the pattern <orgname>-<account_name>.
        account_locator_url : str, optional
            Account URL that is used to connect to the account, in the legacy account locator format.
        comment : str, optional
            Optional comment in which to store information related to the account.
        """
        self.name = name
        self.cloud = cloud
        self.region = region
        self.locator = locator
        self.created_on = created_on
        self.url = url
        self.account_locator_url = account_locator_url
        self.comment = comment
        self.admin_name = admin_name
        self.admin_password = admin_password
        self.account_type = account_type

    __properties = [
        "name",
        "cloud",
        "region",
        "locator",
        "created_on",
        "url",
        "account_locator_url",
        "comment",
        "admin_name",
        "admin_password",
        "account_type",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ManagedAccount(
            name=self.name,
            cloud=self.cloud,
            region=self.region,
            locator=self.locator,
            created_on=self.created_on,
            url=self.url,
            account_locator_url=self.account_locator_url,
            comment=self.comment,
            admin_name=self.admin_name,
            admin_password=self.admin_password,
            account_type=self.account_type,
        )

    @classmethod
    def _from_model(cls, model) -> ManagedAccountModel:
        return ManagedAccountModel(
            name=model.name,
            cloud=model.cloud,
            region=model.region,
            locator=model.locator,
            created_on=model.created_on,
            url=model.url,
            account_locator_url=model.account_locator_url,
            comment=model.comment,
            admin_name=model.admin_name,
            admin_password=model.admin_password,
            account_type=model.account_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ManagedAccount.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ManagedAccount object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ManagedAccountModel:
        """Create an instance of ManagedAccount from a dict.

        This method constructs a ManagedAccount object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ManagedAccount
            A ManagedAccount object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ManagedAccount.from_dict(obj))


ManagedAccount._model_class = ManagedAccountModel



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class StreamingOpenAIReasoning(BaseModel):
    """A model object representing the StreamingOpenAIReasoning resource.

    Constructs an object of type StreamingOpenAIReasoning with the provided properties.

    Parameters
    __________
    summary : str, optional
        Reasoning summary tokens generated during the model response.
    encrypted_content : str, optional
        The encrypted_content field is an opaque field and should not be interpreted or parsed.
        It should only be used for verifying that the reasoning blocks were generated by that specific model.
    """

    summary: Optional[StrictStr] = None

    encrypted_content: Optional[StrictStr] = None

    __properties = ["summary", "encrypted_content"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StreamingOpenAIReasoning:
        """Create an instance of StreamingOpenAIReasoning from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if summary (nullable) is None
        if self.summary is None:
            _dict["summary"] = None

        # set to None if encrypted_content (nullable) is None
        if self.encrypted_content is None:
            _dict["encrypted_content"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> StreamingOpenAIReasoning:
        """Create an instance of StreamingOpenAIReasoning from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return StreamingOpenAIReasoning.model_validate(obj)

        _obj = StreamingOpenAIReasoning.model_validate(
            {
                "summary": obj.get("summary"),
                "encrypted_content": obj.get("encrypted_content"),
            }
        )

        return _obj


class StreamingOpenAIReasoningModel:
    def __init__(
        self,  # optional properties
        summary: Optional[str] = None,
        encrypted_content: Optional[str] = None,
    ):
        """A model object representing the StreamingOpenAIReasoning resource.

                Constructs an object of type StreamingOpenAIReasoning with the provided properties.

                Parameters
                __________
                summary : str, optional
                    Reasoning summary tokens generated during the model response.

                encrypted_content : str, optional
                    The encrypted_content field is an opaque field and should not be interpreted or parsed.
        It should only be used for verifying that the reasoning blocks were generated by that specific model.
        """
        self.summary = summary
        self.encrypted_content = encrypted_content

    __properties = ["summary", "encrypted_content"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return StreamingOpenAIReasoning(
            summary=self.summary,
            encrypted_content=self.encrypted_content,
        )

    @classmethod
    def _from_model(cls, model) -> StreamingOpenAIReasoningModel:
        return StreamingOpenAIReasoningModel(
            summary=model.summary,
            encrypted_content=model.encrypted_content,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a StreamingOpenAIReasoning.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the StreamingOpenAIReasoning object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StreamingOpenAIReasoningModel:
        """Create an instance of StreamingOpenAIReasoning from a dict.

        This method constructs a StreamingOpenAIReasoning object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        StreamingOpenAIReasoning
            A StreamingOpenAIReasoning object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(StreamingOpenAIReasoning.from_dict(obj))


StreamingOpenAIReasoning._model_class = StreamingOpenAIReasoningModel



import pytest

from snowflake.core.exceptions import APIError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_api_error(warehouses, root):
    test_warehouse_name = random_string(3, "test_wh_123")
    new_wh_def = Warehouse(name=test_warehouse_name, warehouse_size="f")
    with pytest.raises(APIError) as exc_info:
        warehouses.create(new_wh_def)
    assert "Error Message: invalid type of property 'f' for 'warehouse_size'" in str(exc_info.value)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
from io import BytesIO
from textwrap import dedent
from time import sleep

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.service import Service, ServiceSpecStageFile

from ..utils import random_string


pytestmark = [pytest.mark.skip_gov]


@pytest.fixture(scope="session")
def seed_temp_service_data():
    return


def test_suspend_resume(root, services, session, imagerepo, shared_compute_pool):
    stage_name = random_string(5, "test_stage_")
    s_name = random_string(5, "test_service_")
    session.sql(f"create temp stage {stage_name};").collect()
    spec_file = "spec.yaml"
    stage_file = f"@{stage_name}"
    spec = f"{stage_file}/{spec_file}"
    image_path = f"{imagerepo}/nginx:latest"
    session.file.put_stream(
        BytesIO(
            dedent(
                f"""
                spec:
                  containers:
                  - name: web-server
                    image: {image_path}
                 """
            ).encode()
        ),
        spec,
    )
    test_s = Service(
        name=s_name,
        compute_pool=shared_compute_pool,
        spec=ServiceSpecStageFile(stage=stage_name, spec_file=spec_file),
        min_instances=1,
        max_instances=1,
    )

    ready_status = ("READY", "DONE", "PENDING")  # fake status might have DONE as status

    s = services.create(test_s)
    try:
        last_status = ""
        for _ in range(10):
            web_server = next(s.get_containers())
            last_status = web_server.service_status
            if last_status in ready_status:
                break
            sleep(1)
        else:
            pytest.fail(f"web_server never came online: {last_status}")
        services[test_s.name].suspend()
        for _ in range(10):
            web_server = next(s.get_containers())
            if web_server.service_status in ("SUSPENDED", "SUSPENDING"):
                break
            sleep(1)
        else:
            pytest.fail("web_server never went to sleep")
        services[test_s.name].resume()
        for _ in range(60):
            web_server = next(s.get_containers())
            if web_server.service_status in ready_status:
                break
            sleep(1)
        else:
            pytest.fail("web_server never resumed")
    finally:
        s.drop()

    with pytest.raises(NotFoundError):
        services["RANDOM"].suspend()
    with pytest.raises(NotFoundError):
        services["RANDOM"].resume()
    services["RANDOM"].suspend(if_exists=True)
    services["RANDOM"].resume(if_exists=True)



from snowflake.core.password_policy import PasswordPolicy


test_password_policy_template = PasswordPolicy(
    name="to_be_set",
    password_min_length=8,
    password_max_length=256,
    password_min_upper_case_chars=1,
    password_min_lower_case_chars=1,
    password_min_numeric_chars=1,
    password_min_special_chars=1,
    password_min_age_days=1,
    password_max_age_days=90,
    password_max_retries=5,
    password_lockout_time_mins=15,
    password_history=24,
    comment="Test password policy",
)

test_password_policy_minimal_template = PasswordPolicy(
    name="to_be_set",
)



import pytest

from snowflake.core.exceptions import ConflictError


pytestmark = [pytest.mark.min_sf_ver("8.35.0")]


def test_create_pipe(pipes, template_pipe):
    try:
        pipe_resource = pipes.create(template_pipe)
        fetched_pipe = pipe_resource.fetch()
        assert fetched_pipe.name == template_pipe.name.upper()
        assert fetched_pipe.comment == template_pipe.comment

        # create pipe with the same name
        with pytest.raises(ConflictError):
            pipes.create(template_pipe)

        # create pipe with the same name but with if_not_exists
        pipes.create(template_pipe, mode="if_not_exists")

        # create pipe with the same name but with or_replace
        template_pipe.comment = "new comment"
        pipe_resource = pipes.create(template_pipe, mode="or_replace")
        fetched_pipe = pipe_resource.fetch()
        assert fetched_pipe.comment == template_pipe.comment

        # use the fetch to create with or_replace
        pipe_resource = pipes.create(fetched_pipe, mode="or_replace")

    finally:
        pipe_resource.drop(if_exists=True)



from collections.abc import Iterator

import pytest

from snowflake.core.dynamic_table import (
    DownstreamLag,
    DynamicTable,
    DynamicTableCollection,
    DynamicTableColumn,
    DynamicTableResource,
)
from snowflake.core.table import Table, TableColumn, TableResource
from tests.utils import random_string


pytestmark = pytest.mark.usefixtures("setup_rest_api_parameters_for_dynamic_table")


@pytest.fixture(scope="module")
def dynamic_tables(schema) -> DynamicTableCollection:
    return schema.dynamic_tables


@pytest.fixture
def table_handle(schema) -> Iterator[TableResource]:
    table_name = '"' + random_string(5, "test_table_") + '"'
    test_table = schema.tables.create(
        Table(
            name=table_name,
            columns=[TableColumn(name="c1", datatype="int"), TableColumn(name="c2", datatype="varchar")],
        )
    )
    try:
        yield test_table
    finally:
        test_table.drop()


@pytest.fixture
def dynamic_table_handle(dynamic_tables, db_parameters, table_handle) -> Iterator[DynamicTableResource]:
    table_name = random_string(10, "test_dynamic_table_")
    test_table = dynamic_tables.create(
        DynamicTable(
            name=table_name,
            warehouse=db_parameters["warehouse"],
            target_lag=DownstreamLag(),
            columns=[DynamicTableColumn(name="a"), DynamicTableColumn(name="b", datatype="varchar", comment="comment")],
            query=f"SELECT * FROM {table_handle.name}",
            initialize="ON_SCHEDULE",
            cluster_by=["b"],
            data_retention_time_in_days=1,
            max_data_extension_time_in_days=2,
            comment="test table",
        ),
        mode="errorifexists",
    )
    try:
        yield test_table
    finally:
        test_table.drop()



# mypy: ignore-errors
from concurrent.futures import Future
from typing import TYPE_CHECKING, Any, Literal, Optional, Union, overload

from snowflake.core import PollingOperation

from .._internal.telemetry import api_telemetry
from .._utils import cast_result
from ..exceptions import InvalidArgumentsError, InvalidResultError
from . import ReturnTable
from ._generated import UserDefinedFunctionArgument
from ._generated.api.user_defined_function_api_base import (
    UserDefinedFunctionCollectionBase,
    UserDefinedFunctionResourceBase,
)


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class UserDefinedFunctionCollection(UserDefinedFunctionCollectionBase):
    """Represents the collection operations on the Snowflake User Defined Function resource.

    With this collection, you can create, iterate through, and fetch user defined functions
    that you have access to in the current context.

    Examples
    ________
    Creating a user defined function instance of python language:

    >>> user_defined_functions.create(
    ...     UserDefinedFunction(
    ...         name="my_python_function",
    ...         arguments=[],
    ...         return_type=ReturnDataType(datatype="VARIANT"),
    ...         language_config=PythonFunction(runtime_version="3.13", packages=[], handler="udf"),
    ...         body='''
    ... def udf():
    ...     return {"key": "value"}
    ...             ''',
    ...     )
    ... )
    """

    _identifier_requires_args = True

    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, UserDefinedFunctionResource)


class UserDefinedFunctionResource(UserDefinedFunctionResourceBase):
    """Represents a reference to a Snowflake user defined function.

    With this user defined function reference, you can fetch information about a user defined
    function, as well as perform certain actions on it.
    """

    _identifier_requires_args = True
    _plural_name = "user_defined_functions"

    def __init__(self, name_with_args: str, collection_class: UserDefinedFunctionCollection) -> None:
        super().__init__(name_with_args, collection_class)

    @api_telemetry
    def rename(
        self,
        target_name: str,
        target_database: Optional[str] = None,
        target_schema: Optional[str] = None,
        if_exists: Optional[bool] = None,
    ) -> None:
        """Rename this user defined function.

        Parameters
        __________
        target_name: str
            The new name of the user defined function
        target_database: str, optional
            The database where the user defined function will be located
        target_schema: str, optional
            The schema where the user defined function will be located
        if_exists: bool, optional
            Check the existence of user defined function before rename

        Examples
        ________
        Renaming this user defined function using its reference:

        >>> user_defined_function_reference.rename("my_other_user_defined_function")

        Renaming this user defined function if it exists:

        >>> user_defined_function_reference.rename("my_other_user_defined_function", if_exists=True)

        Renaming this user defined function and relocating it to another schema within same database:

        >>> user_defined_function_reference.rename(
        ...     "my_other_user_defined_function", target_schema="my_other_schema", if_exists=True
        ... )

        Renaming this user defined function and relocating it to another database and schema:

        >>> user_defined_function_reference.rename(
        ...     "my_other_user_defined_function",
        ...     target_database="my_other_database",
        ...     target_schema="my_other_schema",
        ...     if_exists=True,
        ... )
        """
        if target_database is None:
            target_database = self.database.name
        if target_schema is None:
            target_schema = self.schema.name

        super().rename(
            target_name=target_name,
            target_database=target_database,
            target_schema=target_schema,
            if_exists=if_exists,
        )

    @api_telemetry
    def rename_async(
        self,
        target_name: str,
        target_database: Optional[str] = None,
        target_schema: Optional[str] = None,
        if_exists: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`rename`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        if target_database is None:
            target_database = self.database.name
        if target_schema is None:
            target_schema = self.schema.name

        return super().rename_async(
            target_name=target_name,
            target_database=target_database,
            target_schema=target_schema,
            if_exists=if_exists,
        )

    @api_telemetry
    def execute(self, input_args: Optional[list[UserDefinedFunctionArgument]] = None) -> Any:
        """Execute this user defined function.

        Parameters
        __________
        input_args: list[UserDefinedFunctionArgument], optional
            A list of arguments to pass to the function. The number of arguments must match the number of arguments
            the user defined function expects. Name, datatype and value fields of UserDefinedFunctionArgument are
            required.

        Examples
        ________
        Executing a user defined function using its reference:

        >>> user_defined_function_reference.execute(
        ...     [
        ...         UserDefinedFunctionArgument(name="id", datatype="INT", value=42),
        ...         UserDefinedFunctionArgument(name="tableName", datatype="TEXT", value="my_table_name"),
        ...     ]
        ... )
        """
        return self._execute(input_args, async_req=False)

    @api_telemetry
    def execute_async(self, input_args: Optional[list[UserDefinedFunctionArgument]] = None) -> PollingOperation[Any]:
        """An asynchronous version of :func:`execute`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return self._execute(input_args=input_args, async_req=True)

    @overload
    def _execute(
        self, input_args: Optional[list[UserDefinedFunctionArgument]], async_req: Literal[True]
    ) -> PollingOperation[Any]: ...

    @overload
    def _execute(self, input_args: Optional[list[UserDefinedFunctionArgument]], async_req: Literal[False]) -> Any: ...

    def _execute(
        self, input_args: Optional[list[UserDefinedFunctionArgument]], async_req: bool
    ) -> Union[Any, PollingOperation[Any]]:
        user_defined_function = self.fetch()
        if isinstance(user_defined_function.return_type, ReturnTable):
            raise NotImplementedError("Executing User Defined Table Functions (UDTFs) is not supported.")

        if input_args is None:
            input_args = []

        for arg in user_defined_function.arguments:
            if arg.default_value is None:
                if not any(arg.name.upper() == input_arg.name.upper() for input_arg in input_args):
                    raise InvalidArgumentsError(f"Required argument '{arg.name}' not provided")

        result_or_future = self.collection._api.execute_user_defined_function(
            self.database.name,
            self.schema.name,
            user_defined_function.name,
            user_defined_function_argument=input_args,
            async_req=async_req,
        )

        def map_result(result: object) -> Any:
            if not isinstance(result, dict) or len(result.values()) != 1:
                raise InvalidResultError(f"User Defined Function result {str(result)} is invalid or empty")

            result = list(result.values())[0]
            return cast_result(result, str(user_defined_function.return_type.to_dict().get("datatype")))

        if isinstance(result_or_future, Future):
            return PollingOperation(result_or_future, map_result)
        return map_result(result_or_future)



from typing import TYPE_CHECKING

from ._generated.api.notification_integration_api_base import (
    NotificationIntegrationCollectionBase,
    NotificationIntegrationResourceBase,
)


if TYPE_CHECKING:
    from snowflake.core._root import Root


class NotificationIntegrationCollection(NotificationIntegrationCollectionBase):
    """Represents the collection operations on the Snowflake Notification Integration resource.

    With this collection, you can create, update, and iterate through notification integrations that you have access
    to in the current context.

    Examples
    ________
    Creating a notification integrations instance:

    >>> # This example assumes that mySecret already exists
    >>> notification_integrations = root.notification_integrations
    >>> new_ni = NotificationIntegration(
    ...     name="my_notification_integration",
    ...     enabled=True,
    ...     notification_hook=NotificationWebhook(
    ...         webhook_url="https://events.pagerduty.com/v2/enqueue",
    ...         webhook_secret=WebhookSecret(
    ...             name="mySecret".upper(), database_name=database, schema_name=schema
    ...         ),
    ...         webhook_body_template='{"key": "SNOWFLAKE_WEBHOOK_SECRET", "msg": "SNOWFLAKE_WEBHOOK_MESSAGE"}',
    ...         webhook_headers={"content-type": "application/json", "user-content": "chrome"},
    ...     ),
    ... )
    >>> notification_integrations.create(new_ni)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, NotificationIntegrationResource)


class NotificationIntegrationResource(NotificationIntegrationResourceBase):
    """Represents a reference to a Snowflake Notification Integration resource.

    With this notification integration reference, you can fetch information about a notification
    integration, as well as perform certain actions on it.
    """

    _plural_name = "notification_integrations"
